unit sCustomButton;
(************************************************************************
sStyle components package 
Copyright (c) 2002 Serge Goncharov
                            Ukraine, Odessa
                            http://www.geocities.com/sstylish2002/
                            Serge@HyBce.com
 Version:	1.1, 04-02-2002
*************************************************************************)

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, sStyleUtil,
    sMessages, sConst, ExtCtrls, sPanel, sGraphUtils, commctrl, Buttons, Imglist,
    sUtils, ActnList;

type
  TsCustomButton = Class;

  TsButtonActionLink = class(TControlActionLink)
  protected
    FClient: TsCustomButton;
    procedure AssignClient(AClient: TObject); override;
    function IsCheckedLinked: Boolean; override;
    procedure SetChecked(Value: Boolean); override;
    function IsImageIndexLinked: Boolean; override;
    procedure SetImageIndex(Value: Integer); override;
  end;

  TsCustomButton = class(TCustomControl)
  private

    FSpacing: integer;
    FMarginH: integer;
    FMarginV: integer;
    FImgWidth: integer;
    FImgHeight: integer;
    FAutoSize: boolean;
    procedure SetLayout(const Value: TButtonLayout);
    procedure SetBevelWidth(const Value: integer);
    procedure SetSpacing(const Value: integer);
    procedure SetMarginH(const Value: integer);
    procedure SetMarginV(const Value: integer);
    procedure SetImgHeight(const Value: integer);
    procedure SetImgWidth(const Value: integer);
    procedure SetDown(const Value: boolean);
    procedure SetAutoSize(const Value: boolean);
  protected
    FBevelWidth : integer;
    FDown : boolean;
    FImageIndex : TImageIndex;
    FLayout : TButtonLayout;
    FsStyle : TsPassiveBGStyle;
    FWallpaperCheck: boolean;
    procedure AdjustSize; override;
    procedure ActionChange(Sender: TObject; CheckDefaults: Boolean); override;
    procedure DrawCaption;
    function GetActionLinkClass: TControlActionLinkClass; override;
    function GetDown : integer;
    function GetImgHeight: integer; dynamic;
    function GetImgWidth: integer; dynamic;
    function GetImgRect: TRect; dynamic;
    procedure MouseDown (Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp (Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure SetImageIndex(const Value: TImageIndex);
    procedure WndProc (var Message: TMessage); override;
    procedure WMEraseBkGND (var Message: TWMPaint); message WM_ERASEBKGND;
    procedure WriteText(R : TRect);
//    function DoCanResize(var NewWidth, NewHeight: Integer): Boolean; override;
  public
    constructor Create (AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Paint; override;
    procedure Invalidate; override;
    procedure DrawContents; dynamic;
    procedure DrawGlyph; dynamic;
    property Down : boolean read FDown write SetDown default False;
    property ImgWidth : integer read FImgWidth write SetImgWidth;
    property ImgHeight : integer read FImgHeight write SetImgHeight;
  published
    property AutoSize : boolean read FAutoSize write SetAutoSize default False;
    property sStyle : TsPassiveBGStyle read FsStyle write FsStyle;
    property WallpaperCheck: boolean read FWallpaperCheck write FWallpaperCheck default False;
    property Layout : TButtonLayout read FLayout write SetLayout;
    property BevelWidth : integer read FBevelWidth write SetBevelWidth default 1;
    property Action;
    property Align;
    property Anchors;
//    property AutoSize;
    property Caption;
    property Constraints;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property Font;
    property ImageIndex : TImageIndex read FImageIndex write SetImageIndex default -1;
    property MarginH : integer read FMarginH write SetMarginH default 5;
    property MarginV : integer read FMarginV write SetMarginV default 3;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Spacing : integer read FSpacing write SetSpacing default 3;
    property Visible;
    property OnCanResize;
    property OnClick;
    property OnConstrainedResize;
    property OnContextPopup;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnResize;
    property OnStartDock;
    property OnStartDrag;
  end;


  TsSpeedButton = class(TsCustomButton)
  private
    procedure SetImages(const Value: TCustomImageList);
  protected
    FImages : TCustomImageList;
    FImagesHot : TCustomImageList;
    FImagesDisabled : TCustomImageList;
    procedure DrawGlyph; override;
    function GetImageList : TCustomImageList;
    function GetImgHeight: integer; override;
    function GetImgWidth: integer; override;
    procedure ActionChange(Sender: TObject; CheckDefaults: Boolean); override;
  public
    constructor Create (AOwner: TComponent); override;
  published
    property Down;
    property ImagesDisabled : TCustomImageList read FImagesDisabled write FImagesDisabled;
    property ImagesHot : TCustomImageList read FImagesHot write FImagesHot;
    property Images : TCustomImageList read FImages write SetImages;
  end;

  TsColorSelect = class(TsCustomButton)
  private
    FColorValue : TColor;
    FOnChange : TNotifyEvent;
    procedure SetColorValue(const Value: TColor);
    procedure DrawGlyph; override;
    procedure MouseUp (Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
  protected
  public
    constructor Create (AOwner: TComponent); override;
  published
    property ColorValue : TColor read FColorValue write SetColorValue;
    property ImgWidth : integer read FImgWidth write SetImgWidth;
    property ImgHeight : integer read FImgHeight write SetImgHeight;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  end;

implementation


{ TToolButtonActionLink }

procedure TsButtonActionLink.AssignClient(AClient: TObject);
begin
  inherited AssignClient(AClient);
  FClient := AClient as TsCustomButton;
end;

function TsButtonActionLink.IsCheckedLinked: Boolean;
begin
  Result := inherited IsCheckedLinked and
    (FClient.Down = (Action as TCustomAction).Checked);
end;

function TsButtonActionLink.IsImageIndexLinked: Boolean;
begin
  Result := inherited IsImageIndexLinked{ and
    (FClient.ImageIndex = (Action as TCustomAction).ImageIndex)};
end;

procedure TsButtonActionLink.SetChecked(Value: Boolean);
begin
  if IsCheckedLinked then FClient.Down := Value;
end;

procedure TsButtonActionLink.SetImageIndex(Value: Integer);
begin
  if IsImageIndexLinked then FClient.ImageIndex := Value;
end;

{ TsCustomButton }

procedure TsCustomButton.ActionChange(Sender: TObject;
  CheckDefaults: Boolean);
begin
  if Caption <> TAction(Action).Caption then begin
    Caption := TAction(Action).Caption;
  end
  else inherited;
end;

constructor TsCustomButton.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := [csCaptureMouse, csSetCaption, csClickEvents];
  sStyle := TsPassiveBGStyle.Create(TWinControl(Self));
  sStyle.COC := COC_TsCustomPanel;
  sStyle.GroupIndex := 0;
  sStyle.HotTrack := True;
  ParentFont := True;
  sStyle.Painting.Transparency := 100;
  Height := 34;
  Width := 114;
  FImageIndex := -1;
  FSpacing := 3;
  FMarginH := 5;
  FMarginV := 3;
  FBevelWidth := 1;
//  sStyle.Background.FMode := bgsTransparent;
end;

destructor TsCustomButton.Destroy;
begin
  sStyle.Free;
  inherited;
end;

procedure TsCustomButton.DrawContents;
var
  R: TRect;
  procedure DrawBorder(Down: boolean); begin
    if Down then begin
      Frame3D(sStyle.FCacheBmp.Canvas, R, sStyle.Painting.ColorBorderTop, sStyle.Painting.ColorBorderBottom, BevelWidth);
    end
    else begin
      Frame3D(sStyle.FCacheBmp.Canvas, R, sStyle.Painting.ColorBorderBottom, sStyle.Painting.ColorBorderTop, BevelWidth);
    end;
  end;
begin
  GetWindowRect(Handle, R);
  OffsetRect(R, -R.Left, -R.Top);
  DrawGlyph;
  DrawCaption;
  sStyle.FCacheBmp.Canvas.Pen.Style := psSolid;
  if sStyle.HotTrack then begin
    if sStyle.FMouseAbove or (csDesigning in ComponentState) then begin
      DrawBorder(not Down);
    end;
  end
  else begin
    DrawBorder(not Down);
  end;
end;

function TsCustomButton.GetImgWidth: integer;
begin
  Result := ImgWidth;
end;

function TsCustomButton.GetImgHeight: integer;
begin
  Result := ImgHeight;
end;

procedure TsCustomButton.DrawCaption;
var
  IHnd : cardinal;
  IList: TCustomImageList;
  rText : TRect;
  Enbl : boolean;
  GWidth, GHeight : integer;
begin
  GWidth := GetImgWidth;
  GHeight := GetImgHeight;

  Case Layout of
    blGlyphLeft : begin
      RText.Left := GWidth + Spacing;
      RText.Top := 0;
      RText.Right := ClientRect.Right;
      RText.Bottom := ClientRect.Bottom;
    end;
    blGlyphRight : begin
      RText.Left := 0;
      RText.Top := 0;
      RText.Right := ClientRect.Right - GWidth - Spacing;
      RText.Bottom := ClientRect.Bottom;
    end;
    blGlyphTop : begin
      RText.Left := 0;
      RText.Top := GHeight + Spacing;
      RText.Right := ClientRect.Right;
      RText.Bottom := ClientRect.Bottom;
    end;
    blGlyphBottom : begin
      RText.Left := 0;
      RText.Top := 0;
      RText.Right := ClientRect.Right;
      RText.Bottom := ClientRect.Bottom - GHeight - Spacing;
    end;
  end;
//  OffsetRect(RText, GetDown, GetDown);
  WriteText(RText);
end;

procedure TsCustomButton.DrawGlyph;
begin
  sStyle.FCacheBmp.Canvas.Pen.Style := psClear;
  sStyle.FCacheBmp.Canvas.Brush.Color := Color;
  sStyle.FCacheBmp.Canvas.Rectangle(GetImgRect);
end;

function TsCustomButton.GetActionLinkClass: TControlActionLinkClass;
begin
  Result := TsButtonActionLink;
end;

procedure TsCustomButton.Invalidate;
begin
  AdjustSize;
  inherited;
end;

procedure TsCustomButton.MouseDown(Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if (Button = mbLeft) and Enabled then begin
    Down := True;
    Repaint;
  end;
  inherited;
end;

procedure TsCustomButton.MouseUp(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
begin
  if (Button = mbLeft) and Enabled then begin
    Down := False;
    Repaint;
  end;
  inherited;
end;

procedure TsCustomButton.Paint;
var
  aRect: TRect;
  i: Integer;
  RStep, GStep, BStep: real;
  R, G, B: real;
begin
  if not sStyle.FDrawingStop then begin
    aRect := ClientRect;
    sStyle.FCacheBmp.Height := aRect.Bottom - aRect.Top + 1;
    sStyle.FCacheBmp.Width := aRect.Right - aRect.Left + 1;
    OffsetRect(aRect, GetDown, GetDown);
    if sStyle.Flat then begin
      case sStyle.Background.Mode of
        bgsNone: begin
          Color := sStyle.GetActiveColor;
          sStyle.FCacheBmp.Canvas.Pen.Style := psClear;
          sStyle.FCacheBmp.Canvas.Brush.Style := bsSolid;
          sStyle.FCacheBmp.Canvas.Brush.Color := Color;
          sStyle.FCacheBmp.Canvas.Rectangle(aRect);
        end;
        bgsTransparent: begin
          if sStyle.Painting.Transparency = 0 then begin
            sStyle.FCacheBmp.Canvas.Brush.Style := bsSolid;
            sStyle.FCacheBmp.Canvas.Brush.Color := Color;
            sStyle.FCacheBmp.Canvas.FillRect(aRect);
          end
          else begin
            if (Parent is TsCustomPanel) and Assigned(TsCustomPanel(Parent).sStyle.FCacheBmp) then begin
              if sStyle.Shadow.Enabled and sStyle.Shadow.FLabel.Visible then begin
                // Copying area
                FadeRect(Self, TsCustomPanel(Parent).sStyle.FCacheBmp.Canvas, Rect(Left, Top, Left + Width, Top + Height), sStyle.FCacheBmp.Canvas, Point(0, 0), 100, sStyle.GetActiveColor, 0, ssRectangle);
                // Draw shadow
                FadeRect(Self, sStyle.FCacheBmp.Canvas, Rect(sStyle.Shadow.Offset, sStyle.Shadow.Offset, Width + sStyle.Shadow.Offset, Height + sStyle.Shadow.Offset), sStyle.FCacheBmp.Canvas, Point(sStyle.Shadow.Offset, sStyle.Shadow.Offset), sStyle.Shadow.Transparency, sStyle.Shadow.Color, sStyle.Shadow.Blur, ssRectangle);
                // Fading
                FadeRect(Self, sStyle.FCacheBmp.Canvas, Rect(0, 0, Width, Height), sStyle.FCacheBmp.Canvas, Point(0, 0), sStyle.Painting.Transparency, sStyle.GetActiveColor, 0, ssRectangle);
              end
              else begin
                // Fading
                FadeRect(Self, TsCustomPanel(Parent).sStyle.FCacheBmp.Canvas, Rect(Left, Top, Left + Width, Top + Height), sStyle.FCacheBmp.Canvas, Point(0, 0), sStyle.Painting.Transparency, sStyle.GetActiveColor, 0, ssRectangle);
              end;
            end
            else begin
              sStyle.FCacheBmp.Canvas.Brush.Style := bsSolid;
              sStyle.FCacheBmp.Canvas.Brush.Color := Color;
              sStyle.FCacheBmp.Canvas.FillRect(aRect);
            end;
          end;
        end;
        bgsImage: begin
          if Assigned(sStyle.Background.Image) and Assigned(sStyle.Background.Image.Graphic) and (sStyle.Background.Image.Width <> 0) and (sStyle.Background.Image.Height <> 0) then begin
            TileBitmap(sStyle.FCacheBmp.Canvas, aRect, sStyle.Background.Image.Graphic);
          end;
        end;
        bgsGradient: begin
          case sStyle.Background.Gradient.Layout of
            gtTopBottom, gtLeftRight : begin
              GradientBmp(sStyle.FCacheBmp, ClientRect, sStyle.Background.Gradient.Color1, sStyle.Background.Gradient.Color2, sStyle.Background.Gradient.Layout);
            end;
          end;
        end;
      end;
    end
    else begin
      Color := sStyle.GetActiveColor;
      sStyle.FCacheBmp.Canvas.Pen.Style := psClear;
      sStyle.FCacheBmp.Canvas.Brush.Style := bsSolid;
      sStyle.FCacheBmp.Canvas.Brush.Color := Color;
      sStyle.FCacheBmp.Canvas.Rectangle(Rect(0, 0, sStyle.FCacheBmp.Width, sStyle.FCacheBmp.Height));
    end;
    DrawContents;
    BitBlt(Canvas.Handle, 0, 0, sStyle.FCacheBmp.Width, sStyle.FCacheBmp.Height, sStyle.FCacheBmp.Canvas.Handle, 0, 0, SRCCOPY);
  end;
end;

procedure TsCustomButton.SetBevelWidth(const Value: integer);
begin
  FBevelWidth := Value;
  Invalidate;
end;

procedure TsCustomButton.SetLayout(const Value: TButtonLayout);
begin
  FLayout := Value;
  Invalidate;
end;

procedure TsCustomButton.SetMarginH(const Value: integer);
begin
  FMarginH := Value;
  Invalidate;
end;

procedure TsCustomButton.SetMarginV(const Value: integer);
begin
  FMarginV := Value;
  Invalidate;
end;

procedure TsCustomButton.SetSpacing(const Value: integer);
begin
  FSpacing := Value;
  Invalidate;
end;

procedure TsCustomButton.WMEraseBkGND(var Message: TWMPaint);
begin
  Message.Result := 1;
end;

procedure TsCustomButton.SetImageIndex(const Value: TImageIndex);
begin
  FImageIndex := Value;
  Invalidate;
end;

procedure TsCustomButton.WndProc(var Message: TMessage);
begin
  if Message.Msg = CM_TEXTCHANGED then begin
    inherited;
    Invalidate;
  end
  else begin
    sStyle.WndProc(Message);
    inherited;
  end;
end;

procedure TsCustomButton.WriteText(R: TRect);
const
  Alignments: array[TAlignment] of Longint = (DT_LEFT, DT_RIGHT, DT_CENTER);
var
  Flags: Longint;
begin

  sStyle.FCacheBmp.Canvas.Brush.Style := bsClear;
  sStyle.FCacheBmp.Canvas.Font := Font;

  R.Top := ((R.Bottom + R.Top) - sStyle.FCacheBmp.Canvas.TextHeight('W')) div 2;
  R.Bottom := R.Top + sStyle.FCacheBmp.Canvas.TextHeight('W');

  Flags := DT_EXPANDTABS or DT_VCENTER or Alignments[taCenter];
  Flags := DrawTextBiDiModeFlags(Flags);

  if Enabled then begin
    DrawText(sStyle.FCacheBmp.Canvas.Handle, PChar(Caption), -1, R, Flags);
  end
  else begin
    OffsetRect(r, 1, 1);
    sStyle.FCacheBmp.Canvas.Font.Color := clBtnHighlight;
    DrawText(sStyle.FCacheBmp.Canvas.Handle, PChar(Caption), -1, R, Flags);
    OffsetRect(r, -1, -1);
    sStyle.FCacheBmp.Canvas.Font.Color := clBtnShadow;
    DrawText(sStyle.FCacheBmp.Canvas.Handle, PChar(Caption), -1, R, Flags);
  end;
end;

function TsCustomButton.GetImgRect: TRect;
var
  GWidth, GHeight, x, y : integer;
begin
  GWidth := GetImgWidth;
  GHeight := GetImgHeight;  
  Case Layout of
    blGlyphLeft : begin
      x := MarginH;
      y := (Height - GHeight) div 2;
    end;
    blGlyphRight : begin
      x := Width - MarginH - GWidth;
      y := (Height - GHeight) div 2;
    end;
    blGlyphTop : begin
      x := (Width - GWidth) div 2;
      y := (Height - GHeight - Spacing - Canvas.TextHeight('W')) div 2;
    end;
    blGlyphBottom : begin
      x := (Width - GWidth) div 2;
      y := Height - (Height - GHeight - Spacing - Canvas.TextHeight('W')) div 2 - GHeight;
    end;
  end;
  Result := Rect(x, y, x + GWidth, y + GHeight);
  OffsetRect(Result, GetDown, GetDown);
end;

procedure TsCustomButton.SetImgHeight(const Value: integer);
begin
  FImgHeight := Value;
  Invalidate;
end;

procedure TsCustomButton.SetImgWidth(const Value: integer);
begin
  FImgWidth := Value;
  Invalidate;
end;

procedure TsCustomButton.AdjustSize;
var
  IWidth, IHeight, NewWidth, NewHeight : integer;
begin
  if not (csLoading in ComponentState) and AutoSize then begin
    IWidth := GetImgWidth;
    IHeight := GetImgHeight;
    Inc(IWidth, BevelWidth * 2);
    Inc(IHeight, BevelWidth * 2);
    Case Layout of
      blGlyphLeft, blGlyphRight : begin
        NewWidth := IWidth + Spacing + Canvas.TextWidth(Caption) + MarginH * 2;
        NewHeight := Maxi(IHeight, Canvas.TextHeight('W')) + MarginV * 2;
      end;
      blGlyphTop, blGlyphBottom : begin
        NewWidth := Maxi(IWidth, Canvas.TextWidth(Caption)) + MarginH * 2;
        NewHeight := IHeight + Spacing + Canvas.TextHeight('W') + MarginV * 2;
      end;
    end;
    if NewWidth = 0 then Alert;
    SetBounds(Left, Top, NewWidth, NewHeight);
  end
  else SetBounds(Left, Top, Width, Height);
end;

function TsCustomButton.GetDown: integer;
begin
  if (BevelWidth = 0) and FDown then
    Result := Integer(FDown) * (BevelWidth + 1)
  else
    Result := Integer(FDown) * (BevelWidth);
end;

procedure TsCustomButton.SetDown(const Value: boolean);
begin
  if FDown <> Value then begin
    FDown := Value;
    if sStyle.Shadow.Enabled then begin
      sStyle.BeginDrawing;
      if Value then begin
        SetBounds(Left + sStyle.Shadow.Offset div 3,
                  Top + sStyle.Shadow.Offset div 3,
                  Width,
                  Height);
      end
      else begin
        SetBounds(Left - sStyle.Shadow.Offset div 3,
                  Top - sStyle.Shadow.Offset div 3,
                  Width,
                  Height);
      end;
      sStyle.Shadow.FLabel.Visible := not Value;
      sStyle.EndDrawing;
    end;
    Invalidate;
  end;
end;

{
function TsCustomButton.DoCanResize(var NewWidth, NewHeight: Integer): Boolean;
begin
  Result := False;
end;
}
procedure TsCustomButton.SetAutoSize(const Value: boolean);
begin
  if FAutoSize <> Value then begin
    FAutoSize := Value;
    invalidate;
  end;
end;

{ TsSpeedButton }

constructor TsSpeedButton.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
end;

function TsSpeedButton.GetImageList : TCustomImageList;
begin
  Result := Images;
  if Enabled then begin
    if sStyle.FMouseAbove or not sStyle.HotTrack then begin
      Result := TCustomImageList(iffo(Assigned(ImagesHot), ImagesHot, Images));
    end
    else begin
      Result := Images;
    end;
  end
  else begin
      Result := TCustomImageList(iffo(Assigned(ImagesDisabled), ImagesDisabled, Images));
  end;
end;

function TsSpeedButton.GetImgHeight: integer;
begin
  if GetImageList <> nil then begin
    Result := GetImageList.Height;
  end
  else begin
    Result := 0;
  end
end;

function TsSpeedButton.GetImgWidth: integer;
begin
  if GetImageList <> nil then begin
    Result := GetImageList.Width;
  end
  else begin
    Result := 0;
  end
end;

procedure TsSpeedButton.SetImages(const Value: TCustomImageList);
begin
  FImages := Value;
  Invalidate;
end;

procedure TsSpeedButton.DrawGlyph;
var
  IRect : TRect;
  IList: TCustomImageList;
  Enbl : boolean;
begin
  IList := GetImageList;
  Enbl := Enabled or (IList = ImagesDisabled);
  IRect := GetImgRect;
  
  if Assigned(IList) and (ImageIndex > -1) then begin
    IList.Draw(sStyle.FCacheBmp.Canvas, IRect.Left, IRect.Top, ImageIndex, Enbl);
  end;
end;

procedure TsSpeedButton.ActionChange(Sender: TObject;
  CheckDefaults: Boolean);
begin
  if not Assigned(Images) and Assigned(TAction(Action).ActionList.Images) then begin
    Images := TAction(Action).ActionList.Images;
  end;
  if ImageIndex <> TAction(Action).ImageIndex then begin
    ImageIndex := TAction(Action).ImageIndex;
  end
  else inherited;
end;

{ TsColorSelect }

Constructor TsColorSelect.Create(AOwner: TComponent);
begin
  inherited;
  FImgHeight := 16;
  FImgWidth := 52;
  Height := 28;
  Width := 135
end;

procedure TsColorSelect.DrawGlyph;
begin
  sStyle.FCacheBmp.Canvas.Pen.Style := psClear;
  sStyle.FCacheBmp.Canvas.Brush.Color := ColorValue;
  sStyle.FCacheBmp.Canvas.Rectangle(GetImgRect);
end;

procedure TsColorSelect.MouseUp(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
var
  cd : TColorDialog;
begin
  cd := TColorDialog.Create(Self);
  cd.Color := ColorValue;
  if cd.Execute then begin
    ColorValue := cd.Color;
    if Assigned(FOnChange) then FOnChange(Self);
  end;
  cd.Free;
  inherited;
end;

procedure TsColorSelect.SetColorValue(const Value: TColor);
begin
  FColorValue := Value;
  Invalidate;
end;

end.
