unit sHintManager;
(************************************************************************
sStyle components package 
Copyright (c) 2002 Serge Goncharov
                            Ukraine, Odessa
                            http://www.geocities.com/sstylish2002/
                            Serge@HyBce.com
 Version:	1.1, 04-02-2002
*************************************************************************)

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, sStyleUtil,
  sGraphUtils, sConst, sVclUtils, sUtils, ExtCtrls, sMessages;

type
  THintStyle = (hsRectangle, hsOval);

  TsPassivePaintStyle = class;
  TsHintManager = class;

  TsPassiveShadow = class(TPersistent)
  private
    FEnabled : boolean;
    FColor : TColor;
    FOffset : integer;
    FOwner : TsPassivePaintStyle;
    FTransparency : integer;
    FBlur : integer;

    procedure SetColor(const Value: TColor);
    procedure SetEnabled(const Value: boolean);
    procedure SetOffset(const Value: integer);
    procedure SetTransparency(const Value: integer);
    procedure SetBlur(const Value: integer);
  public
    {  }
    constructor Create(AOwner : TPersistent);
    {  }
  published
    {   }
    property Transparency: integer read FTransparency write SetTransparency;
    {      }
    property Enabled : boolean read FEnabled write SetEnabled;
    {  }
    property Color : TColor read FColor write SetColor;
    {    }
    property Offset : integer read FOffset write SetOffset default 8;
    property Blur : integer read FBlur write SetBlur default 1;
  end;

  TsPassivePainting = class(TPersistent)
  private
    FColorActive : Graphics.TColor;
    FColorInActive : Graphics.TColor;
    FParentColor : boolean;
    FColorBorderTop : Graphics.TColor;
    FColorBorderBottom : Graphics.TColor;
    FTransparency: integer;
    FOwner : TsPassivePaintStyle;
    FOptimize : boolean;
    procedure SetColors (Index: Integer; Value: Graphics.TColor);
    procedure SetTransparency(const Value: integer);
    procedure SetOptimize(const Value: boolean);
  public
    {  }
    constructor Create(AOwner : TPersistent);

    {  }
//    destructor Destroy; override;
  published
    property Optimize : boolean read FOptimize write SetOptimize;
    property Transparency: integer read FTransparency write SetTransparency;
    {     }
    property ColorBorderTop: Graphics.TColor index 0 read FColorBorderTop write SetColors;
    {     }
    property ColorBorderBottom: Graphics.TColor index 1 read FColorBorderBottom write SetColors;
    {   ,   }
    property ColorActive: Graphics.TColor index 2 read FColorActive write SetColors;
    {   ,    }
    property ColorInActive: Graphics.TColor index 3 read FColorInActive write SetColors;
  end;

  TsPassivePaintStyle = class(TPersistent)
  protected
    FGroupIndex : integer;

    procedure SetFlat(Value : boolean);
  public
    FOwner : TComponent;
    FShadow : TsPassiveShadow;
    FPainting : TsPassivePainting;
    FFlat : boolean;
    FDrawingStop : boolean;
    FCacheBmp : Graphics.TBitmap;
    COC : integer;
    sC : TsStyleControl;
    { , ,     }
    FMouseAbove: Boolean;
    {  }
    constructor Create(AOwner : TComponent); overload;
    {  }
    destructor Destroy; override;
    procedure OwnerInvalidate;
    {    }
    procedure WndProc(var Message: TMessage);
    procedure BeginDrawing;
    procedure EndDrawing;
  published
    {   @link(TsShadow)}
    property Shadow: TsPassiveShadow read FShadow write FShadow;
    { ,   }
    property Painting: TsPassivePainting read FPainting write FPainting;
    {  .   @link(TsStyleControl).  -1,     }
    property GroupIndex: integer read FGroupIndex write FGroupIndex;
    {  .  not Flat,    }
    property Flat : boolean read FFlat write SetFlat;
  end;

  TsHintManager = class(TComponent)
  private
    { Private declarations }
    FsStyle : TsPassivePaintStyle;
    FBorderWidth : integer;
    FBlur: integer;
    FMarginH: integer;
    FMarginV : integer;
    FFont : TFont;
    FStyle : THintStyle;
    FRadius : integer;
    procedure SetFont(const Value: TFont);
  protected
    { Protected declarations }
  public
    { Public declarations }
    FForm: TCustomForm;
    constructor Create (AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Invalidate;
    procedure Assign(Source: TsHintManager); overload;
  published
    { Published declarations }
    property Radius : integer read FRadius write FRadius default 0;
    property sStyle : TsPassivePaintStyle read FsStyle write FsStyle;
    property BorderWidth : integer read FBorderWidth write FBorderWidth default 1;
    property Blur : integer read FBlur write FBlur default 0;
    property MarginH : integer read FMarginH write FMarginH default 3;
    property MarginV : integer read FMarginV write FMarginV default 3;
    property Font: TFont read FFont write SetFont;
  end;

  TsHintWindow = class(THintWindow)
  private
    { Private declarations }
//    FManager : TsHintManager;
//    FShowed: Boolean;
  protected
    { Protected declarations }
    procedure Paint; override;
    procedure CreateParams(var Params: TCreateParams); override;
    function CalcHintRect(MaxWidth: Integer; const AHint: string; AData: Pointer): TRect; override;
    procedure WMEraseBkGND (var Message: TWMPaint); message WM_ERASEBKGND;
  public
    { Public declarations }
    procedure ActivateHint(Rect: TRect; const AHint: string); override;
    constructor Create(AOwner:TComponent); override;
  end;

var
  Manager : TsHintManager;

implementation

uses sCustomMenuManager;


constructor TsPassiveShadow.Create(AOwner: TPersistent);
begin
  inherited Create;
  FOwner := TsPassivePaintStyle(AOwner);
  FColor := clBlack;
  FBlur := 1;
  Offset := 8;
end;

procedure TsPassiveShadow.SetBlur(const Value: integer);
begin
  if FBlur <> Value then begin
    FBlur := Value;
  end;
end;

procedure TsPassiveShadow.SetColor(const Value: TColor);
begin
  if (FColor <> Value) then begin
    FColor := Value;
  end;
end;

procedure TsPassiveShadow.SetEnabled(const Value: boolean);
begin
  if (FEnabled <> Value) then begin
    FEnabled := Value;
  end;
end;

procedure TsPassiveShadow.SetOffset(const Value: integer);
begin
  if (FOffset <> Value) then begin
    FOffset := Value;
  end;
end;

procedure TsPassiveShadow.SetTransparency(const Value: integer);
begin
  if FTransparency <> Value then begin
    if Value < 0 then FTransparency := 0
    else if Value > 100 then FTransparency := 100
    else FTransparency := Value;
  end;
end;

constructor TsPassivePainting.Create(AOwner: TPersistent);
begin
  inherited Create;
  FColorBorderTop := clRed;
  FColorBorderBottom := clBlue;
  FColorActive := clYellow;
  FColorInActive := clGray;
  FOptimize := True;
  FOwner := TsPassivePaintStyle(AOwner);
end;

procedure TsPassivePainting.SetColors(Index: Integer; Value: TColor);
begin
  case Index of
    0: FColorBorderTop := Value;
    1: FColorBorderBottom := Value;
    2: FColorActive := Value;
    3: begin
      FColorInActive := Value;
    end;
  end;
end;

procedure TsPassivePainting.SetTransparency(const Value: integer);
begin
  if FTransparency <> Value then begin
    FTransparency := Value;
  end;
end;

procedure TsPassivePainting.SetOptimize(const Value: boolean);
begin
  FOptimize := Value;
end;

constructor TsPassivePaintStyle.Create(AOwner : TComponent);
begin
  inherited Create;
  FOwner := AOwner;
  FCacheBmp := Graphics.TBitmap.Create;

  Shadow := TsPassiveShadow.Create(Self);
  Painting := TsPassivePainting.Create(Self);

  sC := GetsStyleControl(sVclUtils.GetParentForm(AOwner), GroupIndex);
  if not Assigned(sC) then begin
    Painting.FColorActive := clWindow;
    Painting.FColorInActive := cl3DLight;
    Painting.FColorBorderTop := scLightYellow;
    Painting.FColorBorderBottom := scLightYellow;
    FGroupIndex := 0;
    Painting.FParentColor := False;
    Shadow.Color := clBlack;
    Shadow.Offset := 1;
    Shadow.Transparency := 0;
    Shadow.Blur := 4;
    Shadow.Enabled := True;
    FFlat := False;
  end
  else begin
    Painting.FColorActive := sC.ColorActive;
    Painting.FColorInActive := sC.ColorInActive;
    Painting.FColorBorderTop := sC.ColorBorderTop;
    Painting.FColorBorderBottom := sC.ColorBorderBottom;
    FGroupIndex := sC.GroupIndex;
    Painting.FParentColor := sC.ParentColor;
    Shadow.Color := sC.ShadowColor;
    Shadow.Offset := sC.ShadowOffset;
    Shadow.Transparency := sC.ShadowTransparency;
    Shadow.Blur := sC.ShadowBlur;
    Shadow.Enabled := sC.ShadowEnabled;
    Flat := sC.Flat;
  end;
end;

destructor TsPassivePaintStyle.Destroy;
begin
  Painting.Free;
  Shadow.Free;
  inherited;
end;


procedure TsPassivePaintStyle.OwnerInvalidate;
begin
  Case COC of
    COC_TsHintManager : begin
      TsHintManager(FOwner).Invalidate;
    end;
    COC_TsCustomMenuManager, COC_TsMainMenuManager : begin
      TsCustomMenuManager(FOwner).Invalidate;
    end;
  end;
end;

procedure TsPassivePaintStyle.SetFlat(Value : boolean);
begin
  FFlat := Value;
  OwnerInvalidate;
end;

procedure TsPassivePaintStyle.BeginDrawing;
begin
  FDrawingStop := True;
end;

procedure TsPassivePaintStyle.EndDrawing;
begin
  FDrawingStop := False;
//  OwnerInvalidate;
end;

procedure TsPassivePaintStyle.WndProc(var Message: TMessage);
var
  i : integer;
  sSC : TsStyleControl;
begin
  if (Message.Msg >= SM_OFFSET) and (Message.Msg <= SM_LAST) then begin
    if ((Message.WParam = GroupIndex) or (Message.WParam = GI_FORPANELPATTERN)) then begin
//ShowWarning(sMsgToString(Message.Msg) + ', grpIndex=' + IntToStr(Message.WParam));
      case Message.Msg of
        SM_SETFLAT:           Flat := TSMSetBoolean(Message).Value;
        SM_SETBORDERTOP:      Painting.ColorBorderTop := TSMSetColor(Message).Value;
        SM_SETBORDERBOTTOM:   Painting.ColorBorderBottom := TSMSetColor(Message).Value;
        SM_SETCOLORACTIVE:    Painting.ColorActive := TSMSetColor(Message).Value;
        SM_SETCOLORINACTIVE:  Painting.ColorInActive := TSMSetColor(Message).Value;
        SM_SETCOLORSHADOW :   Shadow.Color := TSMSetColor(Message).Value;
        SM_SETPATTERN: begin  end;

//        SM_SETSHADOWENABLED:  Shadow.Enabled := TSMSetBoolean(Message).Value;
//        SM_SHADOWTRANSPARENCY:Shadow.Transparency := TSMSetInteger(Message).Value;
        SM_SHADOWOFFSET:      Shadow.Offset       := TSMSetInteger(Message).Value * 2;
        SM_SHADOWBLUR:        Shadow.Blur         := TSMSetInteger(Message).Value * 2;

        SM_GETSTYLEINFO:      begin
            TSMGetStyleInfo(Message).WParam := 1;
            TSMGetStyleInfo(Message).LParam := Longint(Self);
          end;

        SM_CHANGESTYLECONTROL : begin
            BeginDrawing;
            sSC := TSMChangeStyleControl(Message).sStyleControl;

//            Flat := sSC.Flat;
            Painting.ColorBorderTop := sSC.ColorBorderTop;
            Painting.ColorBorderBottom := sSC.ColorBorderBottom;
            Painting.ColorActive := sSC.ColorActive;
            Painting.ColorInActive := sSC.ColorInActive;
            Shadow.Color := sSC.ShadowColor;

            Shadow.Offset       := sSC.ShadowOffset * 2;
            Shadow.Blur         := sSC.ShadowBlur * 2;
            EndDrawing;
          end;

      end;
    end;
  end;
end;

procedure TsHintManager.Assign(Source: TsHintManager);
begin
  FFont.Assign(Source.Font);
  FBorderWidth := Source.BorderWidth;
  FBlur := Source.Blur;
  FMarginH := Source.MarginH;
  FMarginV := Source.MarginV;
  FRadius := Source.Radius;
  sStyle.Painting.Transparency := Source.sStyle.Painting.Transparency;
  sStyle.Painting.ColorActive := Source.sStyle.Painting.ColorActive;
  sStyle.Painting.ColorBorderTop := Source.sStyle.Painting.ColorBorderTop;
  sStyle.Painting.ColorBorderBottom := Source.sStyle.Painting.ColorBorderBottom;
  sStyle.Shadow.FEnabled := Source.sStyle.Shadow.Enabled;
  sStyle.Shadow.FOffset := Source.sStyle.Shadow.Offset;
  sStyle.Shadow.FTransparency := Source.sStyle.Shadow.Transparency;
  sStyle.Flat := Source.sStyle.Flat;
end;

constructor TsHintManager.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FsStyle := TsPassivePaintStyle.Create(Self);
  FForm := sVclUtils.GetParentForm(AOwner);
  FFont := TFont.Create;
  FFont.Size := 10;
  FFont.Style := [fsBold];
  FFont.Color := scDarkBlue;
  sStyle.COC := COC_TsHintManager;
  FBorderWidth := 1;
  FBlur := 0;
  FMarginH := 3;
  FMarginV := 3;
  FRadius := 0;
  Application.HintHidePause := 4000;
  Application.HintPause := 500;
  Application.HintShortPause := 200;
  sStyle.Painting.Transparency := 0;
  sStyle.Painting.ColorActive := scLightYellow;
  sStyle.Painting.ColorBorderTop := cLRed;
  sStyle.Painting.ColorBorderBottom := cLBlue;
  sStyle.Shadow.Enabled := True;
  sStyle.Shadow.Offset := 14;
  sStyle.Shadow.Transparency := 70;
end;

destructor TsHintManager.Destroy;
begin
  FsStyle.Free;
  FFont.Free;
  inherited;
end;

procedure TsHintManager.Invalidate;
begin
  if Assigned(sStyle) then begin
    if sStyle.Flat then begin
      if not (csDesigning in ComponentState) then begin
        Manager := Self;
        HintWindowClass := TsHintWindow;
      end;
    end
    else begin
      if HintWindowClass = TsHintWindow then begin
        Manager := nil;
        HintWindowClass := THintWindow;
        Alert;
      end;
    end;
  end;
end;

procedure TsHintManager.SetFont(const Value: TFont);
begin
  FFont.Assign(Value);
end;

// Window

procedure TsHintWindow.ActivateHint(Rect: TRect; const AHint: string);
var
  DC: HDC;
begin
  if not Manager.sStyle.FDrawingStop then begin
    Manager.sStyle.BeginDrawing;
//    Application.ProcessMessages;
    ShowWindow(Handle, SW_HIDE);
    Caption := AHint;

    Inc(Rect.Right, (Manager.BorderWidth + Manager.MarginH));
    Inc(Rect.Bottom, (Manager.BorderWidth + Manager.MarginV));
    UpdateBoundsRect(Rect);

    Manager.sStyle.FCacheBmp.Width := Rect.Right - Rect.Left;
    Manager.sStyle.FCacheBmp.Height := Rect.Bottom - Rect.Top;// + 1

    if Rect.Bottom > Screen.DesktopHeight then Rect.Top := Screen.DesktopHeight - Manager.sStyle.FCacheBmp.Height - 2;
    if Rect.Top < Screen.DesktopTop then Rect.Top := Screen.DesktopTop + 2;
    if Rect.Right > Screen.DesktopWidth then Rect.Left := Screen.DesktopWidth - Manager.sStyle.FCacheBmp.Width - 2;
    if Rect.Left < Screen.DesktopLeft then Rect.Left := Screen.DesktopLeft + 2;

    DC := GetDC(0);

    BitBlt(Manager.sStyle.FCacheBmp.Canvas.Handle,
           0,
           0,
           Manager.sStyle.FCacheBmp.Width,
           Manager.sStyle.FCacheBmp.Height,
           DC,
           Rect.Left,
           Rect.Top,
           SrcCopy);

    ReleaseDC(0, DC);

    SetWindowPos(Handle,
                 HWND_TOPMOST,
                 Rect.Left,
                 Rect.Top,
                 Width,
                 Height,
                 SWP_SHOWWINDOW or SWP_NOACTIVATE);
    Invalidate;
    Manager.sStyle.EndDrawing;
  end;
end;

constructor TsHintWindow.Create(AOwner: TComponent);
begin
  inherited;
//  Manager.sStyle.FCacheBmp.Canvas.Font.Assign(Manager.Font);
end;

procedure TsHintWindow.CreateParams(var Params: TCreateParams);
begin
  inherited;
  Params.Style := Params.Style - WS_BORDER + WS_EX_TRANSPARENT;
end;

procedure TsHintWindow.Paint;
var
  DC : HDC;
  P : TPoint;
  R: TRect;
begin
  R := ClientRect;

  // Shadow drawing
  if Manager.sStyle.Shadow.Enabled then begin
    FadeBmp(Manager.sStyle.FCacheBmp,
            Classes.Rect(R.Left + Manager.sStyle.Shadow.Offset,
            R.Top + Manager.sStyle.Shadow.Offset, R.Right, R.Bottom),
            Manager.sStyle.Shadow.Transparency, Manager.sStyle.Shadow.Color, Manager.sStyle.Shadow.Blur, 0);
  end;

  case Manager.sStyle.Painting.Transparency of
    0: begin
      Manager.sStyle.FCacheBmp.Canvas.Brush.Style := bsSolid;
      Manager.sStyle.FCacheBmp.Canvas.Pen.Style := psClear;
      Manager.sStyle.FCacheBmp.Canvas.Brush.Color := Manager.sStyle.Painting.ColorActive;
      Manager.sStyle.FCacheBmp.Canvas.Rectangle(
              Classes.Rect(R.Left, R.Top, R.Right + 1 - iffi(Manager.sStyle.Shadow.Enabled, Manager.sStyle.Shadow.Offset, 0), R.Bottom + 1 - iffi(Manager.sStyle.Shadow.Enabled, Manager.sStyle.Shadow.Offset, 0))
             );
    end;
    else begin
      FadeBmp(Manager.sStyle.FCacheBmp,
              Classes.Rect(R.Left, R.Top, R.Right + 1 - iffi(Manager.sStyle.Shadow.Enabled, Manager.sStyle.Shadow.Offset, 0), R.Bottom + 1 - iffi(Manager.sStyle.Shadow.Enabled, Manager.sStyle.Shadow.Offset, 0)),
              Manager.sStyle.Painting.Transparency, Manager.sStyle.Painting.ColorActive, Manager.Blur, Manager.Radius);
    end;
  end;

  // Text
  InflateRect(R, - Manager.MarginH - Manager.FBorderWidth , - Manager.MarginV - Manager.FBorderWidth);
  inc(R.Top, Manager.MarginV);
  inc(R.Left, Manager.MarginH);

  Manager.sStyle.FCacheBmp.Canvas.Brush.Style := bsClear;
  DrawText(Manager.sStyle.FCacheBmp.Canvas.Handle, PChar(Caption), -1, R, DT_LEFT or DT_NOPREFIX or
    DT_WORDBREAK or DrawTextBiDiModeFlagsReadingOnly);

  R := Classes.Rect(ClientRect.Left,
                    ClientRect.Top,
                    ClientRect.Right - iffi(Manager.sStyle.Shadow.Enabled, Manager.sStyle.Shadow.Offset, 0),
                    ClientRect.Bottom - iffi(Manager.sStyle.Shadow.Enabled, Manager.sStyle.Shadow.Offset, 0));

  if Manager.FBorderWidth > 0 then begin
    Manager.sStyle.FCacheBmp.Canvas.Pen.Style := psSolid;
    Frame3d(Manager.sStyle.FCacheBmp.Canvas,
            R,
            Manager.sStyle.Painting.ColorBorderTop, Manager.sStyle.Painting.ColorBorderBottom, Manager.FBorderWidth);
  end;

  // Copy canvas
  BitBlt(Canvas.Handle,
         0,
         0,
         Manager.sStyle.FCacheBmp.Width,
         Manager.sStyle.FCacheBmp.Height,
         Manager.sStyle.FCacheBmp.Canvas.Handle,
         0,
         0,
         SRCCOPY);
end;

function TsHintWindow.CalcHintRect(MaxWidth: Integer; const AHint: string;
  AData: Pointer): TRect;
begin
  Result := Rect(0, 0, MaxWidth, 0);

  if Assigned(Manager) and Assigned(Manager.Font) then
    Manager.sStyle.FCacheBmp.Canvas.Font.Assign(Manager.Font);

  DrawText(Manager.sStyle.FCacheBmp.Canvas.Handle, PChar(AHint), -1, Result,
           DT_CALCRECT or DT_LEFT or DT_WORDBREAK or DT_NOPREFIX or DrawTextBiDiModeFlagsReadingOnly);

  Inc(Result.Right, (Manager.MarginH + Manager.FBorderWidth) * 2 + iffi(Manager.sStyle.Shadow.Enabled, Manager.sStyle.Shadow.Offset, 0));
  Inc(Result.Bottom, (Manager.MarginV + Manager.FBorderWidth) * 2 + iffi(Manager.sStyle.Shadow.Enabled, Manager.sStyle.Shadow.Offset, 0));
end;

procedure TsHintWindow.WMEraseBkGND(var Message: TWMPaint);
begin
  Message.Result := 1;
end;

end.
