unit sPageControl;
(************************************************************************
sStyle components package 
Copyright (c) 2002 Serge Goncharov
                            Ukraine, Odessa
                            http://www.geocities.com/sstylish2002/
                            Serge@HyBce.com
 Version:	1.1, 04-02-2002
*************************************************************************)

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, sStyleUtil, sConst, ExtCtrls, sPanel, sGraphUtils, sUtils;

type
  TsPageControl = class(TPageControl)
  private
    procedure SetCutSize(const Value: integer);
    { Private declarations }
  protected
    { Protected declarations }
    FsStyle : TsStyle;
    FCutSize : integer;
    procedure WMPaint (var Message: TWMPaint); message WM_PAINT;
    procedure WMNCPaint (var Message: TWMPaint); message WM_NCPAINT;
    procedure WMEraseBkGND (var Message: TWMPaint); message WM_ERASEBKGND;
    procedure WndProc (var Message: TMessage); override;
    procedure Change; override;
  public
    { Public declarations }
    constructor Create(AOwner:TComponent); override;
    procedure Invalidate; override;
    procedure DrawBorders;
    procedure DrawTabs(aRect : Trect);
    property MultiLine;
  published
    { Published declarations }
    property sStyle: TsStyle read FsStyle write FsStyle;
    property CutSize : integer read FCutSize write SetCutSize default 6;
  end;

implementation

{ TsPageControl }

constructor TsPageControl.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  sStyle := TsStyle.Create(Self);
  sStyle.COC := COC_TsTabControl;
  sStyle.Painting.Transparency := 100;
  FCutSize := 6;
end;

procedure TsPageControl.Invalidate;
begin
  if (TabHeight = 0) and Assigned(Parent) then TabHeight := 18;
  if (TabWidth = 0) and Assigned(Parent) then TabWidth := 64;
  if not sStyle.FDrawingStop then begin
    Color := sStyle.Painting.ColorInActive;
  end;
  inherited;
end;

procedure TsPageControl.WMPaint(var Message: TWMPaint);
var
  I, SaveIndex: Integer;
  DC: HDC;
  PS: TPaintStruct;
  R: TRect;
begin
  if sStyle.Flat then begin
    DC := Message.DC;
    if DC = 0 then DC := BeginPaint(Handle, PS);
    SaveIndex := SaveDC(DC);

    if (Parent is TsCustomPanel) and TsCustomPanel(Parent).sStyle.Flat then
      // Copy of parent area
      FadeRect(Self, TsCustomPanel(Parent).sStyle.FCacheBmp.Canvas, Rect(Left, Top, Left + Width, Top + TabHeight + 2), sStyle.FCacheBmp.Canvas, Point(0, 0), 100, {sStyle.Painting.Transparency, }Color, 0, ssRectangle);
    DrawBorders;

    // Copy cash to canvas
    BitBlt(DC, ClientRect.Left, ClientRect.Top, ClientRect.Right - ClientRect.Left, ClientRect.Bottom - ClientRect.Top, sStyle.FCacheBmp.Canvas.Handle, 0, 0, SrcCopy);
    RestoreDC(DC, SaveIndex);
    if Message.DC = 0 then EndPaint(Handle, PS);
    PaintControls(DC, Self);
    Message.Result := 1;
  end
  else Inherited;
end;

procedure TsPageControl.DrawBorders;
var
  R, RText: TRect;
  P1, P2 : TPoint;
  function GetPageX(i : integer) : integer;
  begin
    Result := TabWidth * Pages[i].PageIndex;
  end;
  procedure DrawFrame;
  var
    i, j1, j2 : integer;
    s: string;
  begin
    // Filling working place
    sStyle.FCacheBmp.Canvas.Brush.Color := sStyle.Painting.ColorInActive;// GetActiveColor;
    if (Parent is TsCustomPanel) and sStyle.Flat and TsCustomPanel(Parent).sStyle.Flat then
      sStyle.FCacheBmp.Canvas.FillRect(Rect(R.Left, R.Top + TabHeight + 2, R.Right, R.Bottom))
    else
      sStyle.FCacheBmp.Canvas.FillRect(Rect(0, 0, Width, Height));

    DrawTabs(R);
    // Draw shadow
    for i := 0 to PageCount - 1 do begin
      if ActivePageIndex = i then begin
        if sStyle.Shadow.Enabled then begin
          FadeRect(Self,
                 sStyle.FCacheBmp.Canvas,
                 Rect(TabWidth + GetPageX(i) + 1,
                      sStyle.Shadow.Offset + CutSize,
                      TabWidth + GetPageX(i) + sStyle.Shadow.Offset + 1,
                      TabHeight + 1),

                 sStyle.FCacheBmp.Canvas,
                 Point(TabWidth + GetPageX(i) + 1,
                       sStyle.Shadow.Offset + CutSize),
                 sStyle.Shadow.Transparency,
                 sStyle.Shadow.Color, sStyle.Shadow.Blur, ssRectangle);
        end;
        Break;
      end
    end;
  end;
begin
  GetWindowRect(Handle, R);
  OffsetRect(R, -R.Left, -R.Top);
  sStyle.FCacheBmp.Height := R.Bottom - R.Top;
  sStyle.FCacheBmp.Width := R.Right - R.Left;

  DrawFrame;
  // Outer border - Top
  sStyle.FCacheBmp.Canvas.Pen.Color := sStyle.Painting.ColorBorderTop;
  sStyle.FCacheBmp.Canvas.Polyline([Point(1, R.Bottom - 1),
                                   Point(1, TabHeight + 3)]);

  sStyle.FCacheBmp.Canvas.Pen.Color := LightingColor(sStyle.Painting.ColorBorderTop, 2);
  sStyle.FCacheBmp.Canvas.Polyline([Point(0, R.Bottom - 2),
                                   Point(0, TabHeight + 2)]);

  // Outer border - Bottom
  sStyle.FCacheBmp.Canvas.Pen.Color := DarkingColor(sStyle.Painting.ColorBorderBottom, 2);
  sStyle.FCacheBmp.Canvas.Polyline([Point(R.Right - 1, R.Top + TabHeight + 3),
                   Point(R.Right - 1, R.Bottom - 1),
                   Point(1, R.Bottom - 1)]);

  sStyle.FCacheBmp.Canvas.Pen.Color := sStyle.Painting.ColorBorderBottom;
  sStyle.FCacheBmp.Canvas.Polyline([Point(R.Right - 2, TabHeight + 4),
                   Point(R.Right - 2, R.Bottom - 2),
                   Point(2, R.Bottom - 2)]);

end;

procedure TsPageControl.Change;
begin
  inherited;
  Invalidate;
end;

procedure TsPageControl.WMEraseBkGND(var Message: TWMPaint);
begin
  if sStyle.Flat then begin
    Parent.Perform(WM_PAINT, Message.DC, 0);
    Message.Result := 1;
  end
  else inherited;
end;

procedure TsPageControl.SetCutSize(const Value: integer);
begin
  if FCutSize <> Value then begin
    if Value < 0 then FCutSize := 0
    else if Value > TabHeight then FCutSize := TabHeight
    else FCutSize := Value;
    Invalidate;
  end;
end;

procedure TsPageControl.WndProc(var Message: TMessage);
begin
  inherited;
  sStyle.WndProc(Message);
end;
procedure TsPageControl.WMNCPaint(var Message: TWMPaint);
begin
//  if sStyle.Flat then Message.Result := 1 else
    inherited;
end;

procedure TsPageControl.DrawTabs(aRect : Trect);
var
  i, j1, j2 : integer;
  RText : TRect;
  function GetPageX(i : integer) : integer;
  begin
    Result := TabWidth * Pages[i].PageIndex;
  end;
begin
  if ActivePageIndex <> -1 then begin
    // Drawing tabs
    for i := 0 to PageCount - 1 do begin
      if ActivePageIndex <> i then begin // if not active
        sStyle.FCacheBmp.Canvas.Brush.Color := sStyle.Painting.ColorInActive;

        sStyle.FCacheBmp.Canvas.Pen.Color := sStyle.Painting.ColorBorderBottom;
        sStyle.FCacheBmp.Canvas.PolyLine([Point(GetPageX(i) + 1, aRect.Top + TabHeight + 1),
                        Point(GetPageX(i) + 1, aRect.Top + CutSize div 2 + 2),
                        Point(GetPageX(i) + CutSize div 2, aRect.Top + 2),
                        Point(GetPageX(i) + TabWidth - CutSize div 2, aRect.Top + 2),
                        Point(GetPageX(i) + TabWidth, aRect.Top + CutSize div 2 + 2),
                        Point(GetPageX(i) + TabWidth, aRect.Top + TabHeight + 2)]);

{
        sStyle.FCacheBmp.Canvas.Pen.Color := LightingColor(sStyle.Painting.ColorBorderBottom, 2);
        sStyle.FCacheBmp.Canvas.PolyLine([Point(GetPageX(i) + 2, aRectTop + TabHeight + 1),
                        Point(GetPageX(i) + 2, aRectTop + CutSize div 2 + 2),
                        Point(GetPageX(i) + CutSize div 2 + 1, aRectTop + 3),
                        Point(GetPageX(i) + TabWidth - CutSize div 2, aRectTop + 3),
                        Point(GetPageX(i) + TabWidth - 1, aRectTop + CutSize div 2 + 2),
                        Point(GetPageX(i) + TabWidth - 1, aRectTop + TabHeight + 2)]);
}
      end
      else begin // if active
        sStyle.FCacheBmp.Canvas.Brush.Color := sStyle.Painting.ColorInActive; //sStyle.GetActiveColor;
        sStyle.FCacheBmp.Canvas.Pen.Color := sStyle.Painting.ColorInActive; //sStyle.GetActiveColor;

        sStyle.FCacheBmp.Canvas.Polygon([Point(aRect.Left + GetPageX(i), aRect.Top + TabHeight + 2),
                        Point(GetPageX(i), aRect.Top + CutSize),
                        Point(GetPageX(i) + CutSize, aRect.Top),
                        Point(GetPageX(i) + TabWidth - CutSize, aRect.Top),
                        Point(GetPageX(i) + TabWidth, aRect.Top + CutSize),
                        Point(GetPageX(i) + TabWidth, aRect.Top + TabHeight + 2)]);

        // Top border
        sStyle.FCacheBmp.Canvas.Pen.Color := LightingColor(sStyle.Painting.ColorBorderTop, 2);
        sStyle.FCacheBmp.Canvas.PolyLine([Point(aRect.Left, aRect.Top + TabHeight + 2),
                         Point(GetPageX(i), aRect.Top + TabHeight + 2),
                         Point(GetPageX(i), aRect.Top + CutSize),
                         Point(GetPageX(i) + CutSize, aRect.Top),
                         Point(GetPageX(i) + TabWidth - CutSize, aRect.Top)]);
        sStyle.FCacheBmp.Canvas.PolyLine([Point(GetPageX(i) + TabWidth, aRect.Top + TabHeight + 2),
                         Point(aRect.Right, aRect.Top + TabHeight + 2)]);

        sStyle.FCacheBmp.Canvas.Pen.Color := sStyle.Painting.ColorBorderTop;
        sStyle.FCacheBmp.Canvas.PolyLine([Point(aRect.Left + 1, aRect.Top + TabHeight + 3),
                         Point(GetPageX(i) + 1, aRect.Top + TabHeight + 3),
                         Point(GetPageX(i) + 1, aRect.Top + CutSize),
                         Point(GetPageX(i) + CutSize, aRect.Top + 1),
                         Point(GetPageX(i) + TabWidth - CutSize, aRect.Top + 1)]);
        sStyle.FCacheBmp.Canvas.PolyLine([Point(GetPageX(i) + TabWidth, TabHeight + 3),
                         Point(aRect.Right - 1, TabHeight + 3)]);

        // Bottom border
        sStyle.FCacheBmp.Canvas.Pen.Color := sStyle.Painting.ColorBorderBottom;
        sStyle.FCacheBmp.Canvas.PolyLine([Point(aRect.Left + GetPageX(i) + TabWidth - CutSize, aRect.Top + 1),
                         Point(GetPageX(i) + TabWidth - 1, aRect.Top + CutSize),
                         Point(GetPageX(i) + TabWidth - 1, aRect.Top + TabHeight + 4)]);

        sStyle.FCacheBmp.Canvas.Pen.Color := DarkingColor(sStyle.Painting.ColorBorderBottom, 2);
        sStyle.FCacheBmp.Canvas.PolyLine([Point(aRect.Left + GetPageX(i) + TabWidth - CutSize, aRect.Top),
                         Point(GetPageX(i) + TabWidth, aRect.Top + CutSize),
                         Point(GetPageX(i) + TabWidth, aRect.Top + TabHeight + 3)]);

      end;
//      s := Pages[i].Caption;
      RText := Rect(aRect.Left + GetPageX(i),
                    aRect.Top + 3,
                    aRect.Left + GetPageX(i) + TabWidth,
                    aRect.Top + TabHeight);
      j1 := (TabWidth - sStyle.FCacheBmp.Canvas.TextWidth(Pages[i].Caption) - 2) div 2;
      j2 := (TabHeight - sStyle.FCacheBmp.Canvas.TextHeight(Pages[i].Caption) - 2) div 2;
      InflateRect(RText,
                  -j1,
                  -j2);
      sStyle.FCacheBmp.Canvas.Brush.Style := bsClear;
      sStyle.FCacheBmp.Canvas.Font := Font;

      DrawText(sStyle.FCacheBmp.Canvas.Handle, PChar(Pages[i].Caption), -1, RText, 0);

      if (ActivePageIndex = i) and Pages[i].Focused then begin
        sStyle.FCacheBmp.Canvas.DrawFocusRect(RText);
      end;
    end;
  end;
end;

end.
