unit sPanel;
(************************************************************************
sStyle components package 
Copyright (c) 2002 Serge Goncharov
                            Ukraine, Odessa
                            http://www.geocities.com/sstylish2002/
                            Serge@HyBce.com
 Version:	1.1, 04-02-2002
*************************************************************************)

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, sStyleUtil, sConst, sGraphUtils, sUtils, sCheckedControl;

type

  TsPanel = class;

  TsCustomPanel = class(TCustomPanel)
  private
  protected
    FWallpaperCheck: boolean;
    FsStyle : TsPassiveBGStyle;
    FForbidMouse : boolean;
    procedure WndProc (var Message: TMessage); override;
    procedure WMEraseBkGND (var Message: TWMPaint); message WM_ERASEBKGND;
    procedure WriteText(R : TRect);
  public
    constructor Create (AOwner: TComponent); override;
    destructor Destroy; override;
    procedure DrawPanelBorders(Canvas: TCanvas);
    procedure Paint; override;
    procedure Invalidate; override;
  published
    property sStyle : TsPassiveBGStyle read FsStyle write FsStyle;
    property WallpaperCheck: boolean read FWallpaperCheck write FWallpaperCheck default False;
    property ForbidMouse: boolean read FForbidMouse write FForbidMouse default True;
    property Align;
    property Alignment;
    property Anchors;
    property BevelInner;
    property BevelOuter;
    property BevelWidth;
    property BorderWidth;
    property BorderStyle;
    property Caption;
    property Constraints;
    property UseDockManager default True;
    property DockSite;
    property DragCursor;
    property DragKind;
    property DragMode;
    property DockManager;
    property Enabled;
    property FullRepaint;
    property Font;
    property Locked;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnCanResize;
    property OnClick;
    property OnConstrainedResize;
    property OnContextPopup;
    property OnDockDrop;
    property OnDockOver;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnGetSiteInfo;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnResize;
    property OnStartDock;
    property OnStartDrag;
    property OnUnDock;
  end;

  TsPanel = class(TsCustomPanel)
  private
  protected
  public
    constructor Create (AOwner: TComponent); override;
  published
  end;

  TsContainer = class(TsCustomPanel)
  private
  protected
  public
    constructor Create (AOwner: TComponent); override;
  published
    property BorderStyle default bsSingle;
  end;

  TsGlassPanel = class(TsCustomPanel)
  private
    FMask : TPicture;
    procedure SetMask(const Value: TPicture);
    procedure DrawGlassBorders;
  protected
  public
    constructor Create (AOwner: TComponent); override;
    procedure Paint; override;
  published
    property Mask: TPicture read FMask write SetMask;
  end;

implementation

uses sPageControl, sCustomButton;

{ TPanel }

constructor TsCustomPanel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  sStyle := TsPassiveBGStyle.Create(Self);
  sStyle.COC := COC_TsCustomPanel;
  sStyle.GroupIndex := 0;
  sStyle.HotTrack := False;
  FForbidMouse := True;
  FWallpaperCheck := False;
  ParentFont := True;
end;

destructor TsCustomPanel.Destroy;
begin
  sStyle.Free;
  inherited;
end;

procedure TsCustomPanel.Paint;
var
  aRect: TRect;
  i: Integer;
  RStep, GStep, BStep: real;
  R, G, B: real;
begin
  if not sStyle.FDrawingStop then begin
    if sStyle.Flat then begin
      aRect := ClientRect;
      sStyle.FCacheBmp.Height := aRect.Bottom - aRect.Top;
      sStyle.FCacheBmp.Width := aRect.Right - aRect.Left;

      case sStyle.Background.Mode of
        bgsNone: begin
          Color := sStyle.GetActiveColor;
          sStyle.FCacheBmp.Canvas.Pen.Style := psClear;
          sStyle.FCacheBmp.Canvas.Brush.Style := bsSolid;
          sStyle.FCacheBmp.Canvas.Brush.Color := sStyle.GetActiveColor;
          sStyle.FCacheBmp.Canvas.Rectangle(aRect);
        end;
        bgsTransparent: begin
          if sStyle.Painting.Transparency = 0 then begin
            sStyle.FCacheBmp.Canvas.Brush.Style := bsSolid;
            sStyle.FCacheBmp.Canvas.Brush.Color := Color;
            sStyle.FCacheBmp.Canvas.FillRect(aRect);
          end
          else begin
            if (Parent is TsCustomPanel) and Assigned(TsCustomPanel(Parent).sStyle.FCacheBmp) then begin
              if sStyle.Shadow.Enabled then begin
                // Copying area
                FadeRect(Self, TsCustomPanel(Parent).sStyle.FCacheBmp.Canvas, Rect(Left, Top, Left + Width, Top + Height), sStyle.FCacheBmp.Canvas, Point(0, 0), 100, sStyle.GetActiveColor, 0, ssRectangle);
                // Draw shadow
                FadeRect(Self, sStyle.FCacheBmp.Canvas, Rect(sStyle.Shadow.Offset, sStyle.Shadow.Offset, Width + sStyle.Shadow.Offset, Height + sStyle.Shadow.Offset), sStyle.FCacheBmp.Canvas, Point(sStyle.Shadow.Offset, sStyle.Shadow.Offset), sStyle.Shadow.Transparency, sStyle.Shadow.Color, sStyle.Shadow.Blur, ssRectangle);
                // Fading
                FadeRect(Self, sStyle.FCacheBmp.Canvas, Rect(0, 0, Width, Height), sStyle.FCacheBmp.Canvas, Point(0, 0), sStyle.Painting.Transparency, sStyle.GetActiveColor, 0, ssRectangle);
              end
              else begin
                // Fading
                FadeRect(Self, TsCustomPanel(Parent).sStyle.FCacheBmp.Canvas, Rect(Left, Top, Left + Width, Top + Height), sStyle.FCacheBmp.Canvas, Point(0, 0), sStyle.Painting.Transparency, sStyle.GetActiveColor, 0, ssRectangle);
              end;
            end;
          end;
        end;
        bgsImage: begin
          if Assigned(sStyle.Background.Image) and Assigned(sStyle.Background.Image.Graphic) and (sStyle.Background.Image.Width <> 0) and (sStyle.Background.Image.Height <> 0) then begin
            TileBitmap(sStyle.FCacheBmp.Canvas, aRect, sStyle.Background.Image.Graphic);
//            FadeRect(Self, sStyle.FCacheBmp.Canvas, Rect(0, 0, Width, Height), sStyle.FCacheBmp.Canvas, Point(0, 0), sStyle.Painting.Transparency, Color, 0, ssRectangle);
          end;
        end;
        bgsGradient: begin
          case sStyle.Background.Gradient.Layout of
            gtTopBottom, gtLeftRight : begin
              GradientBmp(sStyle.FCacheBmp, ClientRect, sStyle.Background.Gradient.Color1, sStyle.Background.Gradient.Color2, sStyle.Background.Gradient.Layout);
            end;
          end;
        end;
      end;

      WriteText(aRect);
      DrawPanelBorders(sStyle.FCacheBmp.Canvas);
      BitBlt(Canvas.Handle, 0, 0, sStyle.FCacheBmp.Width, sStyle.FCacheBmp.Height, sStyle.FCacheBmp.Canvas.Handle, 0, 0, SRCCOPY);
    end
    else inherited;
    for i := 0 to ControlCount - 1 do begin
      if Controls[i] is TsPageControl then
        TsPageControl(Controls[i]).Repaint // DrawTabs(Rect(0, 0, TsPageControl(Controls[i]).Width, TsPageControl(Controls[i]).Height))
      else if Controls[i] is TsCheckedControl then
        TsCheckedControl(Controls[i]).PaintControl
      else if Controls[i] is TsCustomButton then
        TsCustomButton(Controls[i]).Repaint;
    end;
  end;
end;

procedure TsCustomPanel.DrawPanelBorders(Canvas: TCanvas);
var
  R: TRect;
  procedure DrawFrame(Canvas: TCanvas; Top: boolean);
  begin
    if Top then begin
      Frame3D(Canvas, R, sStyle.Painting.ColorBorderTop, sStyle.Painting.ColorBorderBottom, BevelWidth);
    end
    else begin
      Frame3D(Canvas, R, sStyle.Painting.ColorBorderBottom, sStyle.Painting.ColorBorderTop, BevelWidth);
    end;
  end;
begin
  Canvas.Pen.Style := psSolid;
  GetWindowRect(Handle, R);
  OffsetRect(R, -R.Left, -R.Top);
  if BevelOuter in [bvSpace, bvRaised] then begin
    DrawFrame(Canvas, True and Enabled);
  end
  else if BevelOuter in [bvLowered] then begin
    DrawFrame(Canvas, False or not(Enabled));
  end;
  InflateRect(R, - BorderWidth - BevelWidth, - BorderWidth - BevelWidth);

  if BevelInner in [bvSpace, bvRaised] then begin
    DrawFrame(Canvas, True and Enabled);
  end
  else if BevelInner in [bvLowered] then begin
    DrawFrame(Canvas, False or not(Enabled));
  end;
end;

procedure TsCustomPanel.WMEraseBkGND(var Message: TWMPaint);
begin
  Message.Result := 1;
end;

procedure TsCustomPanel.WriteText(R : TRect);
const
  Alignments: array[TAlignment] of Longint = (DT_LEFT, DT_RIGHT, DT_CENTER);
var
  Flags: Longint;
begin
  sStyle.FCacheBmp.Canvas.Brush.Style := bsClear;
  sStyle.FCacheBmp.Canvas.Font := Font;
  R.Top := ((R.Bottom + R.Top) - sStyle.FCacheBmp.Canvas.TextHeight('W')) div 2;
  R.Bottom := R.Top + sStyle.FCacheBmp.Canvas.TextHeight('W');
  Flags := DT_EXPANDTABS or DT_VCENTER or Alignments[Alignment];
  Flags := DrawTextBiDiModeFlags(Flags);
  DrawText(sStyle.FCacheBmp.Canvas.Handle, PChar(Caption), -1, R, Flags);
end;

procedure TsCustomPanel.WndProc(var Message: TMessage);
begin
  sStyle.WndProc(Message);
  inherited;
end;

{ TsPanel }

constructor TsPanel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  sStyle.COC := COC_TsPanel;
end;

{ TsContainer }

constructor TsContainer.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  sStyle.COC := COC_TsContainer;
//  sStyle.Painting.Transparency := 0;
  BorderStyle := bsSingle;
  BevelOuter := bvNone;
  sStyle.Flat := True;
  WallpaperCheck := True;
end;

{ TsGlassPanel }

constructor TsGlassPanel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  sStyle.COC := COC_TsGlassPanel;
  FMask := TPicture.Create;
end;

procedure TsGlassPanel.SetMask(const Value: TPicture);
begin
  if Assigned(Value) then begin
    FMask.Assign(Value);
    FMask.Bitmap.PixelFormat := pf24bit;
    FMask.Bitmap.Monochrome := True;
  end
  else begin
    FMask.Assign(nil);
  end;
  Invalidate;
end;

procedure TsGlassPanel.Paint;
var
  R: TRect;
  i: Integer;
begin
  if not sStyle.FDrawingStop then begin
    Color := sStyle.GetActiveColor;
    if sStyle.Flat then begin
      R := ClientRect;
      sStyle.FCacheBmp.Height := R.Bottom - R.Top;
      sStyle.FCacheBmp.Width := R.Right - R.Left;

      if sStyle.Painting.Transparency = 0 then begin
        sStyle.FCacheBmp.Canvas.Brush.Color := Color;
        sStyle.FCacheBmp.Canvas.FillRect(R);
      end else if Assigned(sStyle.Background.Image) and Assigned(sStyle.Background.Image.Graphic) and (sStyle.Background.Image.Width <> 0) and (sStyle.Background.Image.Height <> 0) then begin
        TileBitmap(sStyle.FCacheBmp.Canvas, R, sStyle.Background.Image.Graphic);
        if Assigned(Mask) then begin
          FadeByMask(sStyle.FCacheBmp, Mask.Graphic, Rect(0, 0, Width, Height), sStyle.Painting.Transparency, Color);
        end
        else begin
          FadeRect(Self, sStyle.FCacheBmp.Canvas, Rect(0, 0, Width, Height), sStyle.FCacheBmp.Canvas, Point(0, 0), sStyle.Painting.Transparency, Color, 0, ssRectangle);
        end;
      end
      else begin
        if (Parent is TsCustomPanel) and Assigned(TsCustomPanel(Parent).sStyle.FCacheBmp) then begin
          if sStyle.Shadow.Enabled then begin
            // Copying area
            FadeRect(Self, TsCustomPanel(Parent).sStyle.FCacheBmp.Canvas, Rect(Left, Top, Left + Width, Top + Height), sStyle.FCacheBmp.Canvas, Point(0, 0), 100, Color, 0, ssRectangle);
            // Draw shadow
            FadeRect(Self, sStyle.FCacheBmp.Canvas, Rect(sStyle.Shadow.Offset, sStyle.Shadow.Offset, Width + sStyle.Shadow.Offset, Height + sStyle.Shadow.Offset), sStyle.FCacheBmp.Canvas, Point(sStyle.Shadow.Offset, sStyle.Shadow.Offset), sStyle.Shadow.Transparency, sStyle.Shadow.Color, sStyle.Shadow.Blur, ssRectangle);
            // Fading
            FadeRect(Self, sStyle.FCacheBmp.Canvas, Rect(0, 0, Width, Height), sStyle.FCacheBmp.Canvas, Point(0, 0), sStyle.Painting.Transparency, Color, 0, ssRectangle);
          end
          else begin
            if Assigned(Mask) and not Mask.Bitmap.Empty then begin
              FadeRect(Self, TsCustomPanel(Parent).sStyle.FCacheBmp.Canvas, Rect(Left, Top, Left + Width, Top + Height), sStyle.FCacheBmp.Canvas, Point(0, 0), 100, Color, 0, ssRectangle);
              FadeByMask(sStyle.FCacheBmp, Mask.Graphic, Rect(0, 0, Width, Height), sStyle.Painting.Transparency, Color);
            end
            else begin
              FadeRect(Self, TsCustomPanel(Parent).sStyle.FCacheBmp.Canvas, Rect(Left, Top, Left + Width, Top + Height), sStyle.FCacheBmp.Canvas, Point(0, 0), sStyle.Painting.Transparency, sStyle.GetActiveColor, 0, ssRectangle);
            end;
          end;
        end
        else begin
          if True then begin
            sStyle.FCacheBmp.Canvas.Brush.Color := Color;
            sStyle.FCacheBmp.Canvas.FillRect(R);
          end;
        end;

      end;
      WriteText(R);
      DrawGlassBorders;
      Canvas.CopyRect(ClientRect, sStyle.FCacheBmp.Canvas, ClientRect);
    end
    else inherited;
    for i := 0 to ControlCount - 1 do begin
      if Controls[i] is TsPageControl then
        TsPageControl(Controls[i]).Repaint
      else if Controls[i] is TsCheckedControl then
        TsCheckedControl(Controls[i]).DrawCheckText;
    end;
  end;
end;

procedure TsGlassPanel.DrawGlassBorders;
begin
  //
end;

procedure TsCustomPanel.Invalidate;
begin
  if Showing then begin
    inherited;
  end;
end;

end.
