unit sShadowing;
(************************************************************************
sStyle components package 
Copyright (c) 2002 Serge Goncharov
                            Ukraine, Odessa
                            http://www.geocities.com/sstylish2002/
                            Serge@HyBce.com
 Version:	1.1, 04-02-2002
*************************************************************************)

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, sConst;

type
  TsShadowing = class(TGraphicControl)
  private
    { Private declarations }
    FOptimize : boolean;
    FShape : TsShadowingShape;
    FDrawingStop : boolean;
    procedure SetTransparency(const Value: integer);
    procedure SetOptimize(const Value: boolean);
    procedure SetShape(const Value: TsShadowingShape);
    procedure SetBlur(const Value: smallint);
  protected
    { Protected declarations }
  public
    FBlur : smallint;
    FTransparency : integer;
    { Public declarations }
    FCacheBmp : TBitmap;
    constructor Create(AOwner:TComponent); override;
    procedure Paint; override;
    procedure BeginDrawing;
    procedure EndDrawing;
  published
    { Published declarations }
    property Transparency: integer read FTransparency write SetTransparency;
    property Optimize : boolean read FOptimize write SetOptimize;
    property Shape : TsShadowingShape read FShape write SetShape default ssRectangle;
    property Blur : smallint read FBlur write SetBlur default 1;
    property AutoSize;
    property Caption;
    property Align;
    property Anchors;
    property Color;
    property Top;
    property Left;
    property Height;
    property Width;
    property Constraints;
    property Visible;
    property OnClick;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
{
    property BiDiMode;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property Font;
    property ParentBiDiMode;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property OnContextPopup;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnStartDock;
    property OnStartDrag;}
  end;

implementation

uses sPanel, sStyleUtil, sGraphUtils;

{ TsShadowing }

constructor TsShadowing.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle + [csOpaque, csNoStdEvents] - [csCaptureMouse];
  Color := clBlack;
  Height := 34;
  Width := 96;
  FShape := ssRectangle;
  FBlur := 1;
  FTransparency := 50;
  FCacheBmp := TBitMap.Create;
  FOptimize := True;
  ParentFont := True;
  AutoSize := True;
end;

procedure TsShadowing.BeginDrawing;
begin
  FDrawingStop := True;
end;

procedure TsShadowing.EndDrawing;
begin
  FDrawingStop := False;
  Invalidate;
end;

procedure TsShadowing.Paint;
var
  R: TRect;
  P: TPoint;
  Flags: integer;
begin
  if not FDrawingStop then begin
    R := ClientRect;
    FCacheBmp.Height := R.Bottom - R.Top;
    FCacheBmp.Width := R.Right - R.Left;
    if FTransparency = 0 then begin
      FCacheBmp.Canvas.Brush.Color := Color;
      FCacheBmp.Canvas.FillRect(R);
    end
    else begin
      if (Parent is TsCustomPanel) and Assigned(TsCustomPanel(Parent).sStyle.FCacheBmp) and Optimize then begin
        FadeRect(Self, TsCustomPanel(Parent).sStyle.FCacheBmp.Canvas, Rect(Left, Top, Left + Width, Top + Height), FCacheBmp.Canvas, Point(0, 0), FTransparency, Color, Blur, Shape);
      end
      else begin
        Caption := 'Put me on the sContainer, please.';
        FCacheBmp.Canvas.Brush.Color := clBlack;
        FCacheBmp.Canvas.Rectangle(R);
        FCacheBmp.Canvas.Brush.Style := bsClear;
        FCacheBmp.Canvas.Font := Font;
        FCacheBmp.Canvas.Font.Color := clWhite;

        R.Top := ((R.Bottom + R.Top) - FCacheBmp.Canvas.TextHeight('W')) div 2;
        R.Bottom := R.Top + FCacheBmp.Canvas.TextHeight('W');
        Flags := DT_EXPANDTABS or DT_VCENTER or DT_CENTER;
        Flags := DrawTextBiDiModeFlags(Flags);
        DrawText(FCacheBmp.Canvas.Handle, PChar(Caption), -1, R, Flags);
      end;
    end;
    Canvas.CopyRect(ClientRect, FCacheBmp.Canvas, ClientRect);
  end;
end;

procedure TsShadowing.SetBlur(const Value: smallint);
begin
  if FBlur <> Value then begin
    FBlur := Value;
    invalidate;
  End;
end;

procedure TsShadowing.SetOptimize(const Value: boolean);
begin
  FOptimize := Value;
  Invalidate;
end;

procedure TsShadowing.SetShape(const Value: TsShadowingShape);
begin
  if FShape <> Value then begin
    FShape := Value;
    Invalidate;
  end;
end;

procedure TsShadowing.SetTransparency(const Value: integer);
begin
  if FTransparency <> Value then begin
    if Value < 0 then FTransparency := 0
    else if Value > 100 then FTransparency := 100
    else FTransparency := Value;
    Invalidate;
  end;
end;

end.
