unit sStyleUtil;
(************************************************************************
sStyle components package 
Copyright (c) 2002 Serge Goncharov
                            Ukraine, Odessa
                            http://www.geocities.com/sstylish2002/
                            Serge@HyBce.com
 Version:	1.1, 04-02-2002
*************************************************************************)

interface

uses
  Classes, Controls, db, StrUtils, SysUtils, StdCtrls, sSharedIB, Dialogs,
  Graphics, Forms, Messages, dbCtrls, sShadowing, sConst;

type

  TsPaintStyle = class;
  TsStyle = class;
  TsCustomStyle = class;
  TsAllowing = class;
  TsStyleControl = class;
  TsPassiveBackground = class;
  TsBackground = class;

  TPassiveGradient = class(TPersistent)
  private
    FOwner : TsPassiveBackground;
    FColor1 : TColor;
    FColor2 : TColor;
    FLayout   : TGradientTypes;
    procedure SetColors (Index: Integer; Value: TColor);
    procedure SetLayout(const Value: TGradientTypes);
  public
    constructor Create(AOwner: TsPassiveBackground);
  published
    property Color1: TColor index 0 read FColor1 write SetColors default $00D0CDB9;
    property Color2: TColor index 1 read FColor2 write SetColors default clSilver;
    property Layout: TGradientTypes read FLayout write SetLayout   default gtTopBottom;
  end;

  TsPassiveBackground = class(TPersistent)
  private
    FOwner : TsPaintStyle;
    FImage : TPicture;
    FGradient : TPassiveGradient;
    procedure SetImage(const Value: TPicture);
    procedure SetStyle(const Value: TBgStyle);
  public
    FMode : TBgStyle;
    constructor Create(AOwner : TPersistent);
    destructor Destroy; override;
    procedure BkGroundsUpdate; dynamic;
  published
    property Mode : TBgStyle read FMode write SetStyle default bgsGradient;
    property Image : TPicture read FImage write SetImage;
    property Gradient : TPassiveGradient read FGradient write FGradient;
  end;

  TsBackground = class(TsPassiveBackground)
  private
    FOwner : TsPaintStyle;
  public
    constructor Create(AOwner : TsPaintStyle); overload;
    procedure BkGroundsUpdate; override;
  published
  end;

  TsBgControl = class(TsPassiveBackground)
  private
    FOwner : TsStyleControl;
  public
    {  }
    constructor Create(AOwner : TsStyleControl); overload;
    {  }
//    destructor Destroy; override;
    procedure BkGroundsUpdate; override;
  published
  end;

  { ,  ,   }
  TsPainting = class(TPersistent)
  private
    FColorActive : TColor;
    FColorInActive : TColor;
    FParentColor : boolean;
    FColorBorderTop : TColor;
    FColorBorderBottom : TColor;
    FTransparency: integer;
    FOwner : TsCustomStyle;
    FOptimize : boolean;
    procedure SetColors (Index: Integer; Value: TColor);
    procedure SetParentColor (Value: Boolean);
    procedure SetTransparency(const Value: integer);
    procedure SetOptimize(const Value: boolean);
  public
    {  }
    constructor Create(AOwner : TPersistent);
    {  }
//    destructor Destroy; override;
    property Optimize : boolean read FOptimize write SetOptimize;
  published
    property Transparency: integer read FTransparency write SetTransparency;
    {     }
    property ColorBorderTop: TColor index 0 read FColorBorderTop write SetColors;
    {     }
    property ColorBorderBottom: TColor index 1 read FColorBorderBottom write SetColors;
    {   ,   }
    property ColorActive: TColor index 2 read FColorActive write SetColors;
    {   ,    }
    property ColorInActive: TColor index 3 read FColorInActive write SetColors;
    {  True,  @link(ColorInActive)  ,   Parenta}
    property ParentColor: Boolean read FParentColor write SetParentColor default false;
  end;

  { ,  ,   }
  TsData = class(TPersistent)
  private
    FAllowing : TsAllowing;
    FComment : string;
    FField : TField;
    FIsNumber : boolean;
    FOwner : TsStyle;
    procedure SetField(Value : TField);
  public
    {  }
    constructor Create(AOwner : TPersistent);
    {  }
    destructor Destroy; override;
  published
    {  @link(TsAllowing),    }
    property Allowing: TsAllowing read FAllowing write FAllowing;
    {     }
    property Comment:string read FComment write FComment;
    {   ,     }
    property Field: TField read FField write SetField;
    {  -       .    SQL-(   )}
    property IsNumber:boolean read FIsNumber write FIsNumber;
  end;

  { ,       }

  TsShadow = class(TPersistent)
  private
    FEnabled : boolean;
    FColor : TColor;
    FOffset : integer;
    FOwner : TsCustomStyle;
    FTransparency : integer;
    FBlur : integer;
    FDontUse : boolean;

    procedure SetColor(const Value: TColor);
    procedure SetEnabled(const Value: boolean);
    procedure SetOffset(const Value: integer);
    procedure SetTransparency(const Value: integer);
    procedure SetBlur(const Value: integer);
  public
    FLabel : TsShadowing;
    {  }
    constructor Create(AOwner : TPersistent);
    {  }
    destructor Destroy; override;
  published
    {   }
    property Transparency: integer read FTransparency write SetTransparency default 60;
    {      }
    property Enabled : boolean read FEnabled write SetEnabled;
    {  }
    property Color : TColor read FColor write SetColor default clBlack;
    {    }
    property Offset : integer read FOffset write SetOffset default 8;
    property Blur : integer read FBlur write SetBlur default 8;
    property DontUse : boolean read FDontUse write FDontUse;
  end;

  {         .}
  TsStyleControl = class(TComponent)
  private
    FForm : TCustomForm;
    FBackground : TsBgControl;
    FActive : boolean;
    FColorActive : TColor;
    FColorInActive : TColor;
    FColorBorderTop : TColor;
    FColorBorderBottom : TColor;
    FKeysNextControl : string;
    FKeysPrevControl : string;
    FParentColor : boolean;
    FFlat : boolean;
    FHotTrack : boolean;
    FShadowEnabled : boolean;
    FShadowColor : TColor;
    FShadowOffset : integer;
    FShadowTransparency : integer;
    FShadowBlur : integer;
    FGroupIndex : integer;
    FFont: TFont;
    FOnAssign : TNotifyEvent;
    procedure SetBoolean(Index: Integer; Value: boolean);
    procedure SetInteger(Index: Integer; Value: integer);
    procedure SetFonts(Index: Integer; Value: TFont);
    procedure SetColors (Index: Integer; Value: TColor);
    procedure SetString(Index: Integer; Value: string);
  public
    {  }
    constructor Create(AOwner : TComponent); override;
    procedure Assign(sSC: TsStyleControl);
  published

    property OnAssign: TNotifyEvent read FOnAssign write FOnAssign;
    property Background : TsBgControl read FBackground write FBackground;
    {   }
    property ShadowTransparency: integer index 0 read FShadowTransparency write SetInteger;
    {         }
    property ShadowOffset : integer index 1 read FShadowOffset write SetInteger;
    property ShadowBlur : integer index 2 read FShadowBlur write SetInteger;

    {        }
    property ShadowEnabled : boolean index 0 read FShadowEnabled write SetBoolean;
    {        Parenta}
    property ParentColor : boolean index 1 read FParentColor write SetBoolean;
    {  .  not Flat,     - }
    property Flat : boolean index 2 read FFlat write SetBoolean;
    {         }
    property HotTrack: boolean index 3 read FHotTrack write SetBoolean;

    {    }
    property ColorBorderTop : TColor index 0 read FColorBorderTop write SetColors;
    {    }
    property ColorBorderBottom : TColor index 1 read FColorBorderBottom write SetColors;
    {    }
    property ColorActive : TColor index 2 read FColorActive write SetColors;
    {    }
    property ColorInActive : TColor index 3 read FColorInActive write SetColors;
    {         }
    property ShadowColor : TColor index 4 read FShadowColor write SetColors;

    {     }
//    property CaptionFont : TFont index 0 read FCaptionFont write SetFonts;
//    property Font : TFont index 1 read FFont write SetFonts;

    {        ('13, 40')}
    property KeysNextControl : string index 0 read FKeysNextControl write SetString;
    {        ('38')}
    property KeysPrevControl : string index 1 read FKeysPrevControl write SetString;
    {     GroupIndex = GI_FORPANELPATTERN. ,  sStyleControl.GroupIndex=0}
//    property PatternFileName : string index 2 read FPatternFileName write SetString;

    {    .  0 -    }
    property GroupIndex : integer read FGroupIndex write FGroupIndex;
    {   TsStyleControl}
    property Active : boolean read FActive write FActive;
  end;

  {    }
  TsStyleCaption = class(TPersistent)
  private
    FFont : TFont;
    FOwner : TsCustomStyle;
    FText : string;
    FActive : boolean;
    FMaxWidth : integer;
    FLayout : TsCaptionLayout;
    procedure SetText(const Value: string);
    procedure SetActive(const Value: boolean);
    procedure SetFont(const Value: TFont);
    procedure SetMaxWidth(const Value: integer);
    procedure SetLayout(const Value: TsCaptionLayout);
  public
    {  }
    constructor Create(AOwner : TPersistent);
    {  }
    destructor Destroy; override;
  published
    {  }
    property Layout : TsCaptionLayout read FLayout write SetLayout;
    {    }
    property MaxWidth: integer read FMaxWidth write SetMaxWidth;
    {  }
    property Font: TFont read FFont write SetFont;
    {  }
    property Text: string read FText write SetText;
    {  }
    property Active: boolean read FActive write SetActive;
  end;

  { ,       
   SQL-  @link(InitsEdits), @link(GetInsertText),
  @link(GetUpdateText)}
  TsAllowing=class(TPersistent)
  private
    FInsert   : boolean;
    FUpdate   : boolean;
    FAutoInit : boolean;
    FOwner : TsStyle;
    procedure SetInsert(const Value: boolean);
    procedure SetUpdate(const Value: boolean);
  public
    constructor Create(AOwner : TPersistent);
  published
    {       }
    property Insert: boolean read FInsert write SetInsert;
    {       }
    property Update: boolean read FUpdate write SetUpdate;
    {     }
    property AutoInit: boolean read FAutoInit write FAutoInit;
  end;

  { }

  TsPaintStyle=class(TPersistent)
  protected
    FShadow : TsShadow;
    FPainting : TsPainting;
    FCaption : TsStyleCaption;
    FHotTrack : boolean;
    FGroupIndex : integer;
    FFlat : boolean;
    FTheLabel : TLabel;

    procedure SetFlat(Value : boolean);
  public
    FBorderWidth : integer;
    FOwner : TWinControl;
    FDrawingStop : boolean;
    FCacheBmp : TBitmap;
    COC : integer;
    sC : TsStyleControl;
    { , ,   }
    FFocused : boolean;
    { , ,     }
    FMouseAbove: Boolean;
    constructor Create(AOwner : TWinControl); dynamic;
//    constructor Create(AOwner : TComponent); overload;
    {  }
    destructor Destroy; override;
    {      .   not Enabled,
     Result    }
    function GetActiveColor : TColor;
    {    }
    procedure WndProc(var Message: TMessage);
    {  sStyle}
    procedure sStyleMessage(var Message: TMessage);
    {  }
    procedure PaintWindow(Canvas: TCanvas; Left, Top, Width, Height : integer; Text: string; Enabled: boolean; Flags: Longint);
    procedure WriteText(Canvas: TCanvas; Text: PChar; Enabled: boolean; Left, Top, Width, Height : integer; Flags: Longint);
    {  }
    procedure RedrawBorder;
    {    }
    procedure AlignShadow;
    {    }
    procedure AlignLabel;
    procedure BeginDrawing;
    procedure EndDrawing;
  published
    {   @link(TsShadow)}
    property Shadow: TsShadow read FShadow write FShadow;
    { ,   }
    property Painting: TsPainting read FPainting write FPainting;
    {   @link(TsStyleCaption)}
    property Caption: TsStyleCaption read FCaption write FCaption;
    {  }
    property HotTrack: boolean read FHotTrack write FHotTrack;
    {  .   @link(TsStyleControl).  -1,     }
    property GroupIndex: integer read FGroupIndex write FGroupIndex;
    {  .  not Flat,    }
    property Flat : boolean read FFlat write SetFlat;
  end;

  TsPassiveBGStyle=class(TsPaintStyle)
  protected
    FBackground : TsBackground;
  public
    constructor Create(AOwner : TWinControl); override;
  published
    property Background: TsBackground read FBackground write FBackground;
  end;

  TsCustomStyle=class(TsPaintStyle)
  protected
    FDefaultString : string;

    FKeysNextControl : string;
    FKeysPrevControl : string;
    FEditable : boolean;

  public
    { , ,   - }
    LastControl : boolean;
    {  }
    constructor Create(AOwner : TWinControl);
    {  }
    destructor Destroy; override;
    {    }
    procedure WndProc(var Message: TMessage);
    {  sStyle}
    procedure sStyleMessage(var Message: TMessage);
    {    }
    procedure onKeyDown(Sender:TObject; var Key: Word; Shift: TShiftState);
  published
    {       ,  }
    property KeysNextControl:string read FKeysNextControl write FKeysNextControl;
    {       ,  }
    property KeysPrevControl:string read FKeysPrevControl write FKeysPrevControl;
    {      -  @Link(TsCustomComboBox)}
    property DefaultString:string read FDefaultString write FDefaultString;
    {  ,     @link(EnableEditabled)}
    property Editable:boolean read FEditable write FEditable;
  end;

  TsStyle=class(TsCustomStyle)
  protected
    FData : TsData;
  public
    {  }
    constructor Create(AOwner : TWinControl);
    {    }
    procedure WndProc(var Message: TMessage);
    {  sStyle}
    procedure sStyleMessage(var Message: TMessage);
  published
    { ,   }
    property Data: TsData read FData write FData;
  end;

  TsStyleDB=class(TsCustomStyle)
  protected
  public
    {    }
    procedure WndProc(var Message: TMessage);
    {  sStyle}
    procedure sStyleMessage(var Message: TMessage);
//    function GetDataField : string; virtual;
//    function GetDataSource : TDataSource; virtual;
  published
  end;

{     ,  sStyle}
procedure AppBroadCastS(var Message);
{      ,  sStyle}
procedure BroadCastS(Form: TWinControl; var Message);
{ ,    Control  @link(TsStyle)}
function GetStyleInfo(Control : TComponent) : integer;
{ ,      @link(sStyle)}
function StyleIsActive(sStyle : TsCustomStyle) : boolean;
{   @link(sStyle)  }
function GetsStyle(Control : TComponent) : TsCustomStyle;
{  SQL-   ,    ,       }
function GetInsertText(Panel : TWinControl; KeyName, KeyValue : string) : string;
{  SQL-   ,    ,       }
function GetUpdateText(Panel : TWinControl) : string;
{     ,  @link(TsStyle)}
procedure InitsEdits(Panel : TWinControl);
{ /   ,     Editabled}
procedure EnableEditabled(Panel : TWinControl; Value : boolean);
{  @link(TsStyleControl),     }
function GetsStyleControl(Form: TCustomForm; GroupIndex : integer) : TsStyleControl;
procedure ChangeBkGround(Form: TWinControl; Bg : TsBgControl);

implementation

uses sEdit, sIBControls, sMemo, sCurrencyEdit, sControls, sCustomComboBox,
  sBDEControls, sDateEdit, sCheckBox, sComboEdit, sPanel, sCheckedControl,
  sDBComboBox, sRadioButton, sPageControl, sUtils, sGraphUtils, sVclUtils, sMessages,
  sHintManager, Windows, sCustomButton;

procedure ChangeBkGround(Form: TWinControl; Bg : TsBgControl);
var
  M : TSMSetBground;
begin
  M.sBackground := Bg;
  M.GroupIndex := GI_FORPANELPATTERN;
  M.Msg := SM_SETPATTERN;
  M.Result := 0;
  if Assigned(Form) then begin
    BroadCastS(Form, M);
  end
  else begin
    AppBroadCasts(M);
  end;
end;

procedure AppBroadCastS(var Message);
var
  i: integer;
begin
  for i := 0 to Application.ComponentCount-1 do begin
    if Application.Components[i] is TCustomForm then BroadCastS(TWinControl(Application.Components[i]), Message);
  end;
end;

procedure BroadCastS(Form: TWinControl; var Message);
var
  List : TList;
  i : integer;
begin
  TMessage(Message).Result := -222;

  for i := 0 to Form.ComponentCount - 1 do begin
    if Form.Components[i] is TWinControl then begin
      TWinControl(Form.Components[i]).WindowProc(TMessage(Message));
    end
    else if Form.Components[i] is TsHintManager then begin
      TsHintManager(Form.Components[i]).sStyle.WndProc(TMessage(Message));
    end;
  end;
{
  List := TList.Create;
  Form.GetTabOrderList(List);
  if List.Count>0 then begin
    for i:=0 to List.Count-1 do begin
//      if ControlInStyle(TWinControl(List[i])) then begin
        TWinControl(List[i]).WindowProc(TMessage(Message));
//      end;
    end;
  end;
  List.Free;
}
end;

function GetsStyleControl(Form: TCustomForm; GroupIndex : integer) : TsStyleControl;
var
  i : integer;
begin
  Result := nil;
  if Assigned(Form) then begin
    for i := 0 to Form.ComponentCount - 1 do begin
      if (Form.Components[i] is TsStyleControl) and ((TsStyleControl(Form.Components[i]).GroupIndex = GroupIndex) or (TsStyleControl(Form.Components[i]).GroupIndex = 0)) and TsStyleControl(Form.Components[i]).Active then begin
        Result := TsStyleControl(Form.Components[i]);
        exit;
      end;
    end;
  end;
end;

function GetsStyle(Control : TComponent) : TsCustomStyle;
var
  m : TMessage;
begin
  Result := nil;
  if Control is TWinControl then begin
    m.LParam := 0;
    m.WParam := 0;
    m.Msg    := SM_GETSTYLEINFO;
    m.Result := 0;
    TWinControl(Control).WindowProc(m);//, 0, 0);
    Result := TsCustomStyle(m.LParam);
  end;
end;

function StyleIsActive(sStyle : TsCustomStyle) : boolean;
begin
  if sStyle is TsStyle then
     Result := Assigned(sStyle) and Assigned(TsStyle(sStyle).Data.Field)
  else if sStyle is TsStyleDB then
     Result := Assigned(sStyle) and True{Assigned(TDBEdit(sStyle.FOwner).DataSource)  and (TDBEdit(sStyle.FOwner).DataField <> '')}
  else Result := False;
end;

function GetStyleInfo(Control : TComponent) : integer;
var
  m : TMessage;
begin
  Result := 0;
  if Control is TWinControl then begin
    m.LParam := 0;
    m.WParam := 0;
    m.Msg    := SM_GETSTYLEINFO;
    m.Result := 0;
    TWinControl(Control).WindowProc(m);//, 0, 0);
    Result := m.WParam;
  end;
end;

procedure InitsEdits(Panel : TWinControl);
var
  List : TList;
  i : integer;
  mStyle : TsStyle;
begin
  List := TList.Create;
  Panel.GetTabOrderList(List);
  if List.Count>0 then begin
    for i:=0 to List.Count-1 do begin
      if (GetStyleInfo(List[i]) = tos_SSTYLE) then begin
        mStyle := TsStyle(GetsStyle(TWinControl(List[i])));

        if TWinControl(List[i]) is TsCustomComboBox then begin
          TsCustomComboBox(List[i]).Active := True;
        end;

        //      AutoInit
        if StyleIsActive(mStyle) and mStyle.Data.Allowing.AutoInit then begin

          //  Edit
          if Between(GetsStyle(List[i]).COC, COC_TsEdit, COC_TsMemo) then begin
            if TWinControl(List[i]) is TsMemo then
              TsMemo(List[i]).Text := mStyle.Data.Field.AsString
            else
              TsEdit(List[i]).Text := mStyle.Data.Field.AsString;
          end else

          //  ComboBox
          if Between(GetsStyle(List[i]).COC, COC_TsCustomComboBox, COC_TsBDEComboBox) then begin
            if TsCustomComboBox(List[i]).Items.Count > 0 then begin //   
              if mStyle.Data.IsNumber then begin //    
                if TsCustomComboBox(List[i]).CharsInCode > 0 then begin
                  TsCustomComboBox(List[i]).ItemIndex := TsCustomComboBox(List[i]).IndexOfKod(mStyle.Data.Field.AsString);
                end //    ItemIndex
                else if TsCustomComboBox(List[i]).UseItemIndex then begin
                  TsCustomComboBox(List[i]).ItemIndex := mStyle.Data.Field.asInteger;
                end //    
                else begin
                  TsCustomComboBox(List[i]).ItemIndex := TsCustomComboBox(List[i]).Items.IndexOf(mStyle.Data.Field.asString);
                end;
              end //   
              else begin
                  TsCustomComboBox(List[i]).ItemIndex := TsCustomComboBox(List[i]).Items.IndexOf(mStyle.Data.Field.asString);
              end;
            end;
          end else

          //  DateEdito
          if Between(GetsStyle(List[i]).COC, COC_TsDateEdit, COC_TsDateEdit) then begin
            if mStyle.Data.Field.asString='' then begin
              TsDateEdit(List[i]).Date := Date;
              TsDateEdit(List[i]).Text := '  .  .    ';
            end
            else begin
              TsDateEdit(List[i]).Date := mStyle.Data.Field.asDateTime;
            end;
          end else

          //  CheckBox
          if Between(GetsStyle(List[i]).COC, COC_TsCheckedControl, COC_TsRadioButton) then begin
            TsCheckedControl(List[i]).Checked := mStyle.Data.Field.AsInteger = 1;
          end else

          //  CurEdit
          if Between(GetsStyle(List[i]).COC, COC_TsCurrencyEdit, COC_TsCurrencyEdit) then begin
            TsCurrencyEdit(List[i]).Text := mStyle.Data.Field.AsString;
          end;

          //  Panels
          if Between(GetsStyle(List[i]).COC, COC_TsPanel, COC_TsCustomPanel) then begin
            TsCustomPanel(List[i]).Caption := mStyle.Data.Field.AsString;
          end;
        end;
      end;
    end;
  end;
  List.Free;
end;

function GetInsertText(Panel : TWinControl; KeyName, KeyValue : string) : string;
var
  List : TList;
  i : integer;
  s1, s2 : string;
  mStyle : TsStyle;
begin
  List := TList.Create;
  Panel.GetTabOrderList(List);
  if List.Count>0 then begin
    s1 := KeyName;
    s2 := KeyValue;
    for i:=0 to List.Count-1 do begin
      if (GetStyleInfo(List[i]) = 5) then begin
        mStyle := TsStyle(GetsStyle(TWinControl(List[i])));
        //      AutoInit
        if StyleIsActive(mStyle) and mStyle.Data.Allowing.Insert then begin
          s1 := iff((s1 = ''), mStyle.Data.Field.FieldName, s1 + ', ' + mStyle.Data.Field.FieldName);
          s2 := iff((s2 = ''), '', s2 + ', ');

          //  Edit
          if Between(GetsStyle(List[i]).COC, COC_TsEdit, COC_TsMemo) then begin
            if TWinControl(List[i]) is TsMemo then
              s2 := s2 + iff(mStyle.Data.IsNumber, '0' + TsMemo(List[i]).Text, '''' + CorrectString(TsMemo(List[i]).Text) + '''')
            else
              s2 := s2 + iff(mStyle.Data.IsNumber, '0' + TsEdit(List[i]).Text, '''' + CorrectString(TsEdit(List[i]).Text) + '''');
          end else

          //  ComboBox
          if Between(GetsStyle(List[i]).COC, COC_TsCustomComboBox, COC_TsBDEComboBox) then begin
            if mStyle.Data.IsNumber then begin //    
              if TsCustomComboBox(List[i]).CharsInCode > 0 then begin
                s2 := s2 + IntToStr(TsCustomComboBox(List[i]).GetCurrentKod);
              end //    ItemIndex
              else if TsComboBox(List[i]).UseItemIndex then begin
                s2 := s2 + IntToStr(TsCustomComboBox(List[i]).ItemIndex);
              end //    
              else begin
                s2 := s2 + '''' + CorrectString(TsCustomComboBox(List[i]).Text) + '''';
              end;
            end //   
            else begin
              s2 := s2 + '''' + CorrectString(TsCustomComboBox(List[i]).Text) + '''';
            end;
          end else

          //  DateEdito
          if Between(GetsStyle(List[i]).COC, COC_TsDateEdit, COC_TsDateEdit) then begin
            s2 := s2 + iff(TsDateEdit(List[i]).Text='  .  .    ', 'NULL', '''' + TsDateEdit(List[i]).Text + '''');
          end else

          //  CheckBox
          if Between(GetsStyle(List[i]).COC, COC_TsCheckedControl, COC_TsRadioButton) then begin
            s2 := s2 + iff(TsCheckedControl(List[i]).Checked, '1', '0');
          end else

          //  CurEdit
          if Between(GetsStyle(List[i]).COC, COC_TsCurrencyEdit, COC_TsCurrencyEdit) then begin
            s2 := s2 + iff(mStyle.Data.IsNumber, '0' + TsCurrencyEdit(List[i]).Text, '''' + TsCurrencyEdit(List[i]).Text + '''');
          end else

          //  Panels
          if Between(GetsStyle(List[i]).COC, COC_TsPanel, COC_TsCustomPanel) then begin
            s2 := s2 + iff(mStyle.Data.IsNumber, '0' + TsCustomPanel(List[i]).Caption, '''' + TsCustomPanel(List[i]).Caption + '''');
          end;

        end;
      end;
    end;
  end;
  if s1 <> '' then Result := '(' + s1 + ') values (' + s2 + ')';
  List.Free;
end;

function GetUpdateText(Panel : TWinControl) : string;
var
  List : TList;
  i : integer;
  s : string;
  mStyle : TsStyle;
begin
  List := TList.Create;
  Panel.GetTabOrderList(List);
  if List.Count>0 then begin
    s := '';
    for i:=0 to List.Count-1 do begin
      if (GetStyleInfo(List[i]) = 5) then begin
        mStyle := TsStyle(GetsStyle(TWinControl(List[i])));
        if StyleIsActive(mStyle) and mStyle.Data.Allowing.Update then begin
          s := iff((s = ''), '', s + ', ');

          //  Edit
          if Between(GetsStyle(List[i]).COC, COC_TsEdit, COC_TsMemo) then begin
            if TWinControl(List[i]) is TsMemo then
              s := s + mStyle.Data.Field.FieldName + '=' +
                   iff(TsMemo(List[i]).sStyle.Data.IsNumber, '0' + CorrectString(TsMemo(List[i]).Text), '''' + CorrectString(TsMemo(List[i]).Text) + '''')
            else
              s := s + mStyle.Data.Field.FieldName + '=' +
                   iff(TsEdit(List[i]).sStyle.Data.IsNumber, '0' + CorrectString(TsEdit(List[i]).Text), '''' + CorrectString(TsEdit(List[i]).Text) + '''');
          end else

          //  ComboBox
          if Between(GetsStyle(List[i]).COC, COC_TsCustomComboBox, COC_TsBDEComboBox) then begin
            s := s + mStyle.Data.Field.FieldName + '=';
            if mStyle.Data.IsNumber then begin //    
              if TsCustomComboBox(List[i]).CharsInCode > 0 then begin
                s := s + IntToStr(TsCustomComboBox(List[i]).GetCurrentKod);
              end //    ItemIndex
              else if TsCustomComboBox(List[i]).UseItemIndex then begin
                s := s + IntToStr(TsCustomComboBox(List[i]).ItemIndex);
              end //    
              else begin
                s := s + '''' + CorrectString(TsCustomComboBox(List[i]).Text) + '''';
              end;
            end
            else begin
              s := s + '''' + CorrectString(TsCustomComboBox(List[i]).Text) + '''';
            end
          end else

          //  DateEdito
          if Between(GetsStyle(List[i]).COC, COC_TsDateEdit, COC_TsDateEdit) then begin
            s := s + mStyle.Data.Field.FieldName + '=' +
                   iff(TsDateEdit(List[i]).Text='  .  .    ', 'NULL', '''' + TsDateEdit(List[i]).Text + '''');
          end else

          //  CheckBox
          if Between(GetsStyle(List[i]).COC, COC_TsCheckedControl, COC_TsRadioButton) then begin
            s := s + mStyle.Data.Field.FieldName + '=' +
                   iff(TsCheckedControl(List[i]).Checked, '1', '0');
          end else

          //  CurEdit
          if Between(GetsStyle(List[i]).COC, COC_TsCurrencyEdit, COC_TsCurrencyEdit) then begin
            s := s + mStyle.Data.Field.FieldName + '=' +
                   iff(TsCurrencyEdit(List[i]).sStyle.Data.IsNumber, '0' + TsCurrencyEdit(List[i]).Text, '''' + TsCurrencyEdit(List[i]).Text + '''');
          end else

          //  Panels
{          if Between(GetsStyle(List[i]).COC, COC_TsPanel, COC_TsCustomPanel) then begin
              s := s + mStyle.Data.Field.FieldName + '=' +
                   iff(TsCustomPanel(List[i]).sStyle.Data.IsNumber, '0' + CorrectString(TsCustomPanel(List[i]).Caption), '''' + CorrectString(TsCustomPanel(List[i]).Caption) + '''');
          end;}
        end;
      end;
    end;
  end;
  if s <> '' then Result := s;
  List.Free;
end;

procedure EnableEditabled(Panel : TWinControl; Value : boolean);
var
  M : TSMSetBoolean;
begin
  M.Value := Value;
  M.Msg := SM_SETEDITABLED;
  BroadCastS(Panel, M);
end;

constructor TsAllowing.Create(AOwner : TPersistent);
begin
  inherited Create;
  FOwner    := TsStyle(AOwner);
  FAutoInit := True;
  FInsert   := True;
  FUpdate   := True;
end;

procedure TsAllowing.SetInsert(const Value: boolean);
begin
  if Assigned(FOwner.Data.FField) then begin
    case FOwner.Data.FField.FieldKind of //   
      fkData: begin
        if FInsert <> Value then FInsert := Value;
      end;
      fkCalculated, fkLookup: begin
        FInsert := False;
      end;
    end;
  end;
end;

procedure TsAllowing.SetUpdate(const Value: boolean);
begin
  if Assigned(FOwner.Data.FField) then begin
    case FOwner.Data.FField.FieldKind of //   
      fkData: begin
        if FUpdate <> Value then FUpdate := Value;
      end;
      fkCalculated, fkLookup: begin
        FUpdate := False;
      end;
    end;
  end;
end;

// <<<<<<<<< sStyles <<<<<<<<<<<<<<<<<<<< sStyles <<<<<<<<<<<<<<<<<<<< sStyles <<<<<<<<<<<

constructor TsPaintStyle.Create(AOwner : TWinControl);
begin
  inherited Create;
  FOwner := AOwner; 
  FCacheBmp := Graphics.TBitmap.Create;

  Shadow := TsShadow.Create(Self);
  Painting := TsPainting.Create(Self);
  Caption := TsStyleCaption.Create(Self);
  FBorderWidth := 2;
  sC := GetsStyleControl(sVclUtils.GetParentForm(AOwner), GroupIndex);
  if not Assigned(sC) then begin
    FGroupIndex := 0;
    FFlat := True;
  end
  else begin
    Painting.FColorActive := sC.ColorActive;
    Painting.FColorInActive := sC.ColorInActive;
    Painting.FColorBorderTop := sC.ColorBorderTop;
    Painting.FColorBorderBottom := sC.ColorBorderBottom;
    FGroupIndex := sC.GroupIndex;
    FFlat := sC.Flat;
    Painting.FParentColor := sC.ParentColor;
    FHotTrack := sC.HotTrack;
    Shadow.Color := sC.ShadowColor;
    Shadow.Offset := sC.ShadowOffset;
    Shadow.Transparency := sC.ShadowTransparency;
    Shadow.Blur := sC.ShadowBlur;
    Shadow.Enabled := sC.ShadowEnabled;
  end;
end;

destructor TsPaintStyle.Destroy;
begin
  Painting.Free;
  Shadow.Free;
  if Assigned(FTheLabel) then FTheLabel.Free;
  Caption.Free;
  inherited;
end;

procedure TsPaintStyle.SetFlat(Value : boolean);
begin
  if FFlat <> value then begin
    FFlat := Value;
    FOwner.Invalidate;
  end;
end;

function TsPaintStyle.GetActiveColor: TColor;
begin
  if not (FOwner is TsCustomPanel {and TsPanel(FOwner).TabStop}) and
     FOwner.Enabled and
     (not(csDesigning in FOwner.ComponentState) and
                 (((FOwner is TsCustomComboBox) or (FOwner is TsDBComboBox)) and (Screen.ActiveControl = FOwner)) or
                 (FFocused or
                 ((FMouseAbove and FHotTrack) and not(Screen.ActiveControl = FOwner)))) then begin
    Result := Painting.FColorActive
  end
  else
    Result := Painting.FColorInActive;
end;

procedure TsPaintStyle.RedrawBorder;
var
  DC: HDC;
  R: TRect;
  BorderBrush, WinBrush: HBRUSH;
begin
  if not FDrawingStop then begin
    if FFlat then begin
      DC := GetWindowDC(FOwner.Handle);
      try
        GetWindowRect(FOwner.Handle, R);
        OffsetRect(R, -R.Left, -R.Top);
        if (FOwner is TsCustomComboBox) then begin
          R.Right := R.Right - TsCustomComboBox(FOwner).ButtonWidth - 1;
        end;
        if (FOwner is TsDBComboBox) then begin
          R.Right := R.Right - TsDBComboBox(FOwner).ButtonWidth - 1;
        end;

        BorderBrush := CreateSolidBrush(ColorToRGB(Painting.FColorBorderTop));
        WinBrush := CreateSolidBrush(ColorToRGB(GetActiveColor));

        if FOwner.Enabled then
          DrawRectangleOnDC(DC, R, Painting.ColorBorderTop, Painting.ColorBorderBottom, 1)
        else
          DrawRectangleOnDC(DC, R, Painting.ColorBorderBottom, Painting.ColorBorderTop, 1);

        InflateRect(R, -1, -1);
        FrameRect(DC, R, WinBrush);
        InflateRect(R, -1, -1);
        FrameRect(DC, R, WinBrush);
      finally
        ReleaseDC(FOwner.Handle, DC);
      end;
      DeleteObject(WinBrush);
      DeleteObject(BorderBrush);
    end
  end;
end;

procedure TsPaintStyle.AlignShadow;
var
  NewLeft, NewTop, NewWidth, NewHeight : integer;
begin
  if Assigned(Shadow) and Shadow.Enabled and Assigned(Shadow.FLabel) then begin
    if (Fowner is TsCheckedControl) then begin
      case TsCheckedControl(FOwner).Layout of
        calLeft : if Shadow.FLabel.Left <> (FOwner.Left + Shadow.Offset) then begin
          Shadow.FLabel.Left := FOwner.Left + Shadow.Offset;
        end;
        calRight : if Shadow.FLabel.Left <> (FOwner.Left + FOwner.Width + Shadow.Offset - 14) then begin
          Shadow.FLabel.Left := FOwner.Left + FOwner.Width + Shadow.Offset - 14;
        end;
      end;
      if Shadow.FLabel.Top <> (FOwner.Top + Shadow.Offset + 1) then begin
        Shadow.FLabel.Top := FOwner.Top + Shadow.Offset + 1;
      end;
      Shadow.FLabel.Width := 14;
      Shadow.FLabel.Height := 14;
    end
    else begin
      if (Fowner is TsPageControl) then begin
        if Shadow.FLabel.Top <> (FOwner.Top + Shadow.Offset + TsPageControl(FOwner).TabHeight) then
          Shadow.FLabel.Top := FOwner.Top + Shadow.Offset + TsPageControl(FOwner).TabHeight;
        if Shadow.FLabel.Height <> (FOwner.Height - TsPageControl(FOwner).TabHeight) then
          Shadow.FLabel.Height := FOwner.Height - TsPageControl(FOwner).TabHeight;
      end
      else begin
        if Shadow.FLabel.Top <> (FOwner.Top + Shadow.Offset) then
          Shadow.FLabel.Top := FOwner.Top + Shadow.Offset;
        if Shadow.FLabel.Height <> FOwner.Height then
          Shadow.FLabel.Height := FOwner.Height;
      end;
      if Shadow.FLabel.Left <> (FOwner.Left + Shadow.Offset) then
        Shadow.FLabel.Left := FOwner.Left + Shadow.Offset;
      if Shadow.FLabel.Width <> FOwner.Width then
        Shadow.FLabel.Width := FOwner.Width;
    end;
  end;
  if (Fowner is TsCheckedControl) and Assigned(Fowner.Parent) then begin
    TsCheckedControl(Fowner).PaintControl;
  end
  else if (Fowner is TsCustomButton) and Assigned(Fowner.Parent) then begin
    TsCustomButton(Fowner).Invalidate;
  end;
end;

procedure TsPaintStyle.AlignLabel;
begin
  if Assigned(FTheLabel) then begin
    FTheLabel.AutoSize := True;
    case Caption.Layout of
      sclLeft : begin
        FTheLabel.Alignment := taRightJustify;
        FTheLabel.Left := FOwner.Left - FTheLabel.Width - 4;
        FTheLabel.Top := FOwner.Top -
                         (GetFontHeight(FTheLabel.Font.Handle) - (FOwner.Height - 6)) div 2 +
                         6;
      end;
      sclTopLeft : begin
        FTheLabel.Alignment := taLeftJustify;
        FTheLabel.Left := FOwner.Left;
        FTheLabel.Top := FOwner.Top - GetFontHeight(FTheLabel.Font.Handle) + 5;
      end;
      sclTopCenter : begin
        FTheLabel.Alignment := taCenter;
        FTheLabel.Left := FOwner.Left + (FOwner.Width - FTheLabel.Width) div 2;
        FTheLabel.Top := FOwner.Top - GetFontHeight(FTheLabel.Font.Handle) + 5;
      end;
      sclTopRight : begin
        FTheLabel.Alignment := taRightJustify;
        FTheLabel.Left := FOwner.Left + FOwner.Width - FTheLabel.Width;
        FTheLabel.Top := FOwner.Top - GetFontHeight(FTheLabel.Font.Handle) + 5;
      end;
    end;
    if Caption.FMaxWidth <> 0 then begin
      FTheLabel.AutoSize := False;
      FTheLabel.Width := Caption.FMaxWidth;
      FTheLabel.WordWrap := True;
    end;
    FTheLabel.Transparent := True;
  end;
end;

procedure TsPaintStyle.BeginDrawing;
begin
  FDrawingStop := True;
end;

procedure TsPaintStyle.EndDrawing;
begin
  FDrawingStop := False;
  FOwner.Invalidate;
end;

procedure TsPaintStyle.WndProc(var Message: TMessage);
begin
  if (Message.Msg >= SM_OFFSET) and (Message.Msg <= SM_LAST) then begin
    if Message.Msg = SM_GETSTYLEINFO then begin
      TSMGetStyleInfo(Message).WParam := tos_SPAINTSTYLE;
      TSMGetStyleInfo(Message).LParam := Longint(Self);
    end
    else sStyleMessage(Message);
  end
  else begin
    case Message.Msg of
      WM_KILLFOCUS: begin
        FFocused := False;
        FOwner.Invalidate;
      end;
      WM_SETFOCUS: begin
        FFocused := True;
        FOwner.Invalidate;
      end;
      CM_SYSCOLORCHANGE, CM_PARENTCOLORCHANGED: begin
        if Painting.ParentColor and Assigned(FOwner.Parent) then begin
          Painting.FColorInActive := TForm(FOwner.Parent).Color;
        end;
        FOwner.Invalidate;
      end;
      CM_ENABLEDCHANGED, WM_FONTCHANGE: begin
        if Caption.FActive and Assigned(FTheLabel) then FTheLabel.Enabled := FOwner.Enabled;
        FOwner.Invalidate;
      end;
      CM_VISIBLECHANGED: begin
        if Caption.FActive and Assigned(FTheLabel) then FTheLabel.Visible := FOwner.Visible;
        if Shadow.FEnabled and Assigned(Shadow.FLabel) then Shadow.FLabel.Visible := FOwner.Visible;
      end;
      CM_MOUSEENTER : begin
        FMouseAbove := True;
        if not(csDesigning in FOwner.ComponentState) and HotTrack then begin
          if FOwner is TsCheckedControl then begin
            if not TsCheckedControl(FOwner).Focused then begin
              TsCheckedControl(FOwner).DrawCheckArea;
            end;
          end
          else if (FOwner is TsCustomComboBox) and Flat and Assigned(FOwner.Parent) then begin
            TsCustomComboBox(FOwner).Color := GetActiveColor;
            RedrawBorder;
            Message.Result := 1;
          end
          else if (FOwner is TsDBComboBox) and Flat and Assigned(FOwner.Parent) then begin
            TsDBComboBox(FOwner).Color := GetActiveColor;
            RedrawBorder;
            Message.Result := 1;
          end
          else FOwner.Invalidate;
        end;
      end;
      CM_MOUSELEAVE : begin
        FMouseAbove := False;
        if not(csDesigning in FOwner.ComponentState) and HotTrack then begin
          if (FOwner is TsCheckedControl) and not TsCheckedControl(FOwner).Focused then begin
            TsCheckedControl(FOwner).DrawCheckArea
          end
          else if (FOwner is TsCustomComboBox) and Flat and Assigned(FOwner.Parent) then begin
            TsCustomComboBox(FOwner).Color := GetActiveColor;
            RedrawBorder;
            Message.Result := 1;
          end
          else if (FOwner is TsDBComboBox) and Flat and Assigned(FOwner.Parent) then begin
            TsDBComboBox(FOwner).Color := GetActiveColor;
            RedrawBorder;
            Message.Result := 1;
          end
          else FOwner.Invalidate;
        end;
      end;
      WM_SIZE, WM_MOVE : begin
        if Shadow.Enabled and Assigned(Shadow.FLabel) then begin
          AlignShadow;
        end
        else if FOwner is TsCustomButton then TsCustomButton(FOwner).Repaint;
        if Caption.Active then begin
          AlignLabel;
        end;
        if FOwner is TsCheckedControl then TsCheckedControl(FOwner).PaintControl;
      end;
    end;
  end;
end;

procedure TsPaintStyle.sStyleMessage(var Message: TMessage);
var
  i : integer;
  sSC : TsStyleControl;
begin
  if Assigned(Self) and ((Message.WParam = GroupIndex) or (Message.WParam = GI_FORPANELPATTERN)) then begin
    case Message.Msg of
      SM_SETSHADOWENABLED:  Shadow.Enabled := TSMSetBoolean(Message).Value;
      SM_SETFLAT:           Flat := TSMSetBoolean(Message).Value;
      SM_SETPARENTCOLOR:    Painting.ParentColor := TSMSetBoolean(Message).Value;
      SM_SETHOTTRACK:      begin
        if ((FOwner is TsCustomPanel) and TsCustomPanel(FOwner).ForbidMouse) then
          HotTrack := False
        else
          HotTrack := TSMSetBoolean(Message).Value;
      end;
      SM_SETCAPTIONFONT: begin
        Caption.Font.Assign(TFont(TSMSetFont(Message).Value));
        if Caption.Active then begin
          FTheLabel.Font.Assign(TFont(TSMSetFont(Message).Value));
          AlignLabel;
        end;
      end;

      SM_SETBORDERTOP:      Painting.ColorBorderTop := TSMSetColor(Message).Value;
      SM_SETBORDERBOTTOM:   Painting.ColorBorderBottom := TSMSetColor(Message).Value;
      SM_SETCOLORACTIVE:    Painting.ColorActive := TSMSetColor(Message).Value;
      SM_SETCOLORINACTIVE:  begin
        if (FOwner is TsCustomComboBox) then TsCustomComboBox(FOwner).ColorButton := TSMSetColor(Message).Value;
        if (FOwner is TsDBComboBox)     then TsDBComboBox(FOwner).ColorButton := TSMSetColor(Message).Value;
        Painting.ColorInActive := TSMSetColor(Message).Value;
      end;
      SM_SETCOLORSHADOW :   Shadow.Color := TSMSetColor(Message).Value;

      SM_SETPATTERN: begin
            if (FOwner is TsCustomPanel) and TsCustomPanel(FOwner).WallpaperCheck then begin
//              if TSMSetBground(Message).sBackground.Mode = bgsGradient then Alert;
              TsCustomPanel(FOwner).sStyle.Background.Mode := TSMSetBground(Message).sBackground.Mode;
              case TSMSetBground(Message).sBackground.Mode of
                bgsNone: begin
                  TsCustomPanel(FOwner).sStyle.Background.Image.Assign(nil);
                end;
                bgsGradient: begin
                  TsCustomPanel(FOwner).sStyle.Background.Gradient.Color1 := TSMSetBground(Message).sBackground.Gradient.Color1;
                  TsCustomPanel(FOwner).sStyle.Background.Gradient.Color2 := TSMSetBground(Message).sBackground.Gradient.Color2;
//                  TsCustomPanel(FOwner).sStyle.Background.Gradient.Layout := TSMSetBground(Message).sBackground.Gradient.Layout;
                end;
                bgsImage: begin
                  if Assigned(TSMSetBground(Message).sBackground.Image.Bitmap) then begin
                    TsCustomPanel(FOwner).sStyle.Background.Image.Bitmap.Assign(TSMSetBground(Message).sBackground.Image.Bitmap);
                  end;
                end;
              end;
//              if TsCustomPanel(FOwner).Showing then begin
                FOwner.Invalidate;
//              end;
            end;
            Message.Result := 1;
      end;
      SM_SHADOWTRANSPARENCY:Shadow.Transparency := TSMSetInteger(Message).Value;
      SM_SHADOWOFFSET:      Shadow.Offset       := TSMSetInteger(Message).Value;
      SM_SHADOWBLUR:        Shadow.Blur         := TSMSetInteger(Message).Value;

      SM_CHANGESTYLECONTROL : begin
            BeginDrawing;
            sSC := TSMChangeStyleControl(Message).sStyleControl;
            Shadow.Enabled := sSC.ShadowEnabled;
            Flat := sSC.Flat;
            Painting.ParentColor := sSC.ParentColor;

            if ((FOwner is TsCustomPanel) and TsCustomPanel(FOwner).ForbidMouse) then
              HotTrack := False
            else
              HotTrack := sSC.HotTrack;

            Painting.ColorBorderTop := sSC.ColorBorderTop;
            Painting.ColorBorderBottom := sSC.ColorBorderBottom;
            Painting.ColorActive := sSC.ColorActive;

            if (FOwner is TsCustomComboBox) then TsCustomComboBox(FOwner).ColorButton := sSC.ColorInActive;
            Painting.ColorInActive := sSC.ColorInActive;

            Shadow.Color := sSC.ShadowColor;

            if (FOwner is TsCustomPanel) and TsCustomPanel(FOwner).WallpaperCheck and Assigned(sSC.Background) then begin
              TsCustomPanel(FOwner).sStyle.Background.Mode := sSC.Background.Mode;
              case sSC.Background.Mode of
                bgsNone: begin
                  TsCustomPanel(FOwner).sStyle.Background.Image.Assign(nil);
                end;
                bgsImage: begin
                  if Assigned(sSC.Background.Image) then begin
                    TsCustomPanel(FOwner).sStyle.Background.Image.Assign(sSC.Background.Image);
                  end;
                end;
                bgsGradient: begin
                  TsCustomPanel(FOwner).sStyle.Background.Gradient.Color1 := sSC.Background.Gradient.Color1;
                  TsCustomPanel(FOwner).sStyle.Background.Gradient.Color2 := sSC.Background.Gradient.Color2;
//                  TsCustomPanel(FOwner).sStyle.Background.Gradient.Layout := sSC.Background.Gradient.Layout;
                end;
              end;
//              if TsCustomPanel(FOwner).Showing then begin
                FOwner.Invalidate;
//              end;
            end;

            Shadow.Transparency := sSC.ShadowTransparency;
            Shadow.Offset       := sSC.ShadowOffset;
            Shadow.Blur         := sSC.ShadowBlur;
            EndDrawing;
      end;
    end;
  end;
end;

// TsCustomStyle

constructor TsCustomStyle.Create(AOwner : TWinControl);
begin
  inherited Create(AOwner);

  sC := GetsStyleControl(sVclUtils.GetParentForm(AOwner), GroupIndex);
  if not Assigned(sC) then begin
    FKeysNextControl := iff(FOwner is TsMemo, '40', '13, 40');
    FKeysPrevControl := iff(FOwner is TsMemo, '', '38');
  end;
  FEditable := True;
end;

procedure TsCustomStyle.WndProc(var Message: TMessage);
begin
  if (Message.Msg >= SM_OFFSET) and (Message.Msg <= SM_LAST) then begin
    if Message.Msg = SM_GETSTYLEINFO then begin
      TSMGetStyleInfo(Message).WParam := tos_SCUSTOMSTYLE;
      TSMGetStyleInfo(Message).LParam := Longint(Self);
    end else sStyleMessage(Message);
  end
  else inherited;
end;

procedure TsCustomStyle.sStyleMessage(var Message: TMessage);
begin
    if Assigned(Self) and ((Message.WParam = GroupIndex) or (Message.WParam = GI_FORPANELPATTERN)) then begin
      case Message.Msg of
        SM_SETKEYNEXT:        FKeysNextControl := TSMSetString(Message).Value;
        SM_SETKEYPREV:        FKeysPrevControl := TSMSetString(Message).Value;
        SM_SETEDITABLED:      FOwner.Enabled := Editable or TSMSetBoolean(Message).Value;
        else inherited;
      end
    end;
end;

procedure TsCustomStyle.onKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
var
  i, StartIndex : integer;
  List : TList;
  CurControl : TWinControl;
  ToExit : boolean;
begin
{
  if not (ssAlt in Shift) and not (ssShift in Shift) and not (ssCtrl in Shift) then begin
    if not ((FOwner is TsCustomComboBox) and TsComboBox(FOwner).DroppedDown) and
       not ((FOwner is TsDateEdit) and TsDateEdit(FOwner).PopupVisible) then begin
      if StrUtils.IsWordPresent(IntToStr(Key), KeysNextControl, [',']) then begin
        if Assigned(Shared.GetParentForm(FOwner)) then begin
          try
            Shared.GetParentForm(FOwner).Perform(WM_NEXTDLGCTL, 0, 0)
          except
            ShowMessage('Ok');
          end;

        end
//        FOwner.SelectNext(ActiveControl as tWinControl, True, True );
      end
      else if StrUtils.IsWordPresent(IntToStr(Key), KeysPrevControl, [',']) then begin
      end;
    end
  end;
}
  if not (ssAlt in Shift) and not (ssShift in Shift) and not (ssCtrl in Shift) then begin
    if not (( (FOwner is TsCustomComboBox) and TsCustomComboBox(FOwner).DroppedDown
            ) or ((FOwner is TsDBComboBox) and TsDBComboBox(FOwner).DroppedDown)) and
       not ((FOwner is TsDateEdit) and TsDateEdit(FOwner).PopupVisible) then begin
      if StrUtils.IsWordPresent(IntToStr(Key), KeysNextControl, [',']) then begin
        List := TList.Create;
        GetParentForm(FOwner).GetTabOrderList(List);
        if List.Count > 0 then begin
          StartIndex := List.IndexOf(FOwner);
          if StartIndex = -1 then StartIndex := List.Count - 1;
          I := StartIndex;
          ToExit := False;
          repeat
            Inc(I);
            if I = List.Count then I := 0;
            CurControl := List[I];
            if CurControl.CanFocus and CurControl.TabStop and CurControl.Enabled and CurControl.Visible then begin
              LastControl := i = 0;
              if (FOwner is TsCustomComboBox) or (FOwner is TsDBComboBox) then FFocused := False;
              FOwner.Invalidate;
              CurControl.SetFocus;
              ToExit := True;
            end;
          until ToExit or (I = StartIndex);
        end;
        List.Free;
        Key := 0;
      end
      else if StrUtils.IsWordPresent(IntToStr(Key), KeysPrevControl, [',']) then begin
        List := TList.Create;
        GetParentForm(FOwner).GetTabOrderList(List);
        if List.Count > 0 then begin
          StartIndex := List.IndexOf(FOwner);
          if StartIndex = -1 then
            StartIndex := 0;
          I := StartIndex;
          ToExit := False;
          repeat
            if I = 0 then I := List.Count;
            Dec(I);
            CurControl := List[I];
            if CurControl.CanFocus and CurControl.TabStop and CurControl.Enabled and CurControl.Visible then begin
              if (FOwner is TsCustomComboBox) or (FOwner is TsDBComboBox) then FFocused := False;
              FOwner.Invalidate;
              CurControl.SetFocus;
              ToExit := True;
            end;
          until ToExit or (I = StartIndex);
        end;
        List.Free;
        Key := 0;
      end;
    end
  end;
end;

{ TCaption }

constructor TsStyleCaption.Create(AOwner: TPersistent);
begin
  inherited Create;
  FFont := TFont.Create;
  FLayout := sclLeft;
  FOwner := TsCustomStyle(AOwner);
end;

procedure TsStyleCaption.SetActive(const Value: boolean);
begin
  if Value then begin
    if not Active and (StyleIsActive(FOwner) or (Text <> '')) then begin
      FActive := True;
      FOwner.FTheLabel := TLabel.Create(FOwner.FOwner);
      FOwner.FTheLabel.Parent := FOwner.FOwner.Parent;
      FOwner.FTheLabel.ControlStyle := [csOpaque, csReplicatable];
      FOwner.FTheLabel.Layout := tlTop;//tlCenter;

      FOwner.FTheLabel.Alignment := taRightJustify;
      FOwner.FTheLabel.Name := FOwner.FOwner.Name + 'Label';
      if StyleIsActive(FOwner) then begin
        Case GetStyleInfo(FOwner.FOwner) of
          5: begin
            Text := iff(Text = '', TsStyle(FOwner).Data.Field.DisplayLabel, Text);
          end;
          1..4, 6: begin
  //          Text := TDBEdit(FOwner.FOwner).DataSource.DataSet.FieldByName(TDBEdit(FOwner.FOwner).DataField).DisplayLabel;
            Text := iff(Text = '', FOwner.FOwner.Name, Text);
          end;
        end;
      end
      else
        Text := iff(Text = '', FOwner.FOwner.Name, Text);
      FOwner.FTheLabel.Caption := Text;
      FOwner.FTheLabel.Visible := FOwner.FOwner.Visible;
      FOwner.FTheLabel.Enabled := FOwner.FOwner.Enabled;
      FOwner.FTheLabel.ParentFont := True;
      FOwner.AlignLabel;
    end;
  end
  else begin
    if Active then begin
      if Assigned(FOwner.FTheLabel) then FOwner.FTheLabel.Free;
      FActive := False;
    end;
  end;
end;

procedure TsStyleCaption.SetText(const Value: string);
begin
  FText := Value;
  if Active then begin
    FOwner.FTheLabel.Caption := Value;
    FOwner.AlignLabel;
  end;
end;

destructor TsCustomStyle.Destroy;
begin
  inherited;
end;

destructor TsStyleCaption.Destroy;
begin
  FFont.Free;
  inherited;
end;

procedure TsStyleCaption.SetFont(const Value: TFont);
begin
  FFont.Assign(Value);
  if Active then begin
    FOwner.FTheLabel.Font.Assign(Value);
    FOwner.AlignLabel;
  end;
end;

procedure TsStyleCaption.SetMaxWidth(const Value: integer);
begin
  FMaxWidth := Value;
  if Active then begin
    FOwner.AlignLabel;
  end;
end;

procedure TsStyleCaption.SetLayout(const Value: TsCaptionLayout);
begin
  FLayout := Value;
  if Active then begin
    FOwner.AlignLabel;
  end;
end;

{ TsStyleControl }

constructor TsStyleControl.Create(AOwner: TComponent);
begin
  inherited;
  FBackground           := TsBgControl.Create(Self);
//  FCaptionFont          := TFont.Create;
  FFont                 := TFont.Create;

  FKeysNextControl      := '40';
  FKeysPrevControl      := '38';
  FParentColor          := True;
  FColorActive          := clWindow;
  FColorInActive        := cl3DLight;
  FColorBorderTop       := clWhite;
  FColorBorderBottom    := clWhite;
  FHotTrack            := HotTrack;
  FShadowEnabled        := False;
  FShadowColor          := clBlack;
  FShadowOffset         := 8;
  FShadowBlur           := 8;
  FShadowTransparency   := 60;
  FFlat                 := True;
  FActive               := True;
  if not (AOwner is TCustomForm) then FForm := GetParentForm(TWinControl(AOwner)) else FForm :=TCustomForm(AOwner);
end;

procedure TsStyleControl.SetFonts(Index: Integer; Value: TFont);
var
  M : TSMSetFont;
begin
  if FActive then begin
    M.Value := LongWord(Value);
    M.GroupIndex := GroupIndex;
    case index of
      0: M.Msg := SM_SETCAPTIONFONT;
      1: M.Msg := SM_SETFONT;
    end;
    BroadCastS(FForm, M);
  end;
  case index of
//    0: FCaptionFont.Assign(Value);
    1: FFont.Assign(Value);
  end;
end;

procedure TsStyleControl.SetColors(Index: Integer; Value: TColor);
var
  M : TSMSetColor;
begin
  if FActive then begin
    M.Value := Value;
    M.GroupIndex := GroupIndex;
    case index of
      0: M.Msg := SM_SETBORDERTOP;
      1: M.Msg := SM_SETBORDERBOTTOM;
      2: M.Msg := SM_SETCOLORACTIVE;
      3: M.Msg := SM_SETCOLORINACTIVE;
      4: M.Msg := SM_SETCOLORSHADOW;
    end;
    BroadCastS(FForm, M);
  end;
  case index of
    0: FColorBorderTop := Value;
    1: FColorBorderBottom := Value;
    2: FColorActive := Value;
    3: begin
      FParentColor := (FColorInActive <> Value) and not FParentColor;
      FColorInActive := Value;
    end;
    4: FShadowColor := Value;
  end;
end;

procedure TsStyleControl.SetBoolean(Index: Integer; Value: boolean);
var
  M : TSMsetBoolean;
begin
  if FActive then begin
    M.Value := Value;
    M.GroupIndex := GroupIndex;
    case Index of
      0: M.Msg := SM_SETSHADOWENABLED;
      1: M.Msg := SM_SETPARENTCOLOR;
      2: M.Msg := SM_SETFLAT;
      3: M.Msg := SM_SETHotTrack;
    end;
    BroadCastS(FForm, M);
  end;
  case Index of
    0: FShadowEnabled := Value;
    1: begin
      FParentColor := Value;
      if Value then FColorInActive := FForm.Color;
    end;
    2: FFlat := Value;
    3: FHotTrack := Value;
  end;
end;

procedure TsStyleControl.SetInteger(Index, Value: integer);
var
  M : TSMSetInteger;
begin
  if FActive then begin
    M.Value := Value;
    M.GroupIndex := GroupIndex;
    case Index of
      0: M.Msg := SM_SHADOWTRANSPARENCY;
      1: M.Msg := SM_SHADOWOFFSET;
      2: M.Msg := SM_SHADOWBLUR;
    end;
    BroadCastS(FForm, M);
  end;
  Case Index of
    0: FShadowTransparency := Value;
    1: FShadowOffset := Value;
    2: FShadowBlur := Value;
  end;
end;

procedure TsStyleControl.SetString(Index: Integer; Value: string);
var
  M : TSMSetString;
begin
  if FActive then begin
    M.Value := Value;
    M.GroupIndex := GroupIndex;
    case Index of
      0: {M.Msg := SM_SETKEYNEXT};
      1: {M.Msg := SM_SETKEYPREV};
{      2: if (GroupIndex = 0) and (FileExists(Value) or (Value = '')) then begin
           M.GroupIndex := GI_FORPANELPATTERN;
           M.Msg := SM_SETPATTERN;
         end else
           M.Msg := WM_NULL;}
    end;
    BroadCastS(FForm, M);
  end;
  case Index of
    0: FKeysNextControl := Value;
    1: FKeysPrevControl := Value;
//    2: if (GroupIndex = 0) and (FileExists(Value) or (Value = '')) then FPatternFileName := Value;
  end;
end;

procedure TsStyleControl.Assign(sSC: TsStyleControl);
var
  M : TSMChangeStyleControl;
begin
  if Assigned(sSC) and (GroupIndex = sSC.GroupIndex) then begin
    Active := False;
//    Self.Background.SetStyle(sSC.Background.Mode);
//    Background.Image.Assign(sSC.Background.Image);
//    ChangeBkGround(FForm, sSC.BackGround);

    M.sStyleControl := sSC;
    M.GroupIndex := GroupIndex;
    M.Msg := SM_CHANGESTYLECONTROL;
    BroadCastS(FForm, M);

    Active := False;
    ShadowBlur := sSc.ShadowBlur;
    ColorActive := sSC.ColorActive;
    ColorInActive := sSC.ColorInActive;
    ColorBorderTop := sSC.ColorBorderTop;
    ColorBorderBottom := sSC.ColorBorderBottom;
    Flat := sSC.Flat;
    FParentColor := sSC.ParentColor;
    HotTrack := sSC.HotTrack;
    ShadowColor := sSC.ShadowColor;
    ShadowOffset := sSC.ShadowOffset;
    ShadowTransparency := sSC.ShadowTransparency;
    ShadowEnabled := sSC.ShadowEnabled;

    Active := True;
    if Assigned(OnAssign) then OnAssign(Self);

  end;

end;

{ TsShadow }

constructor TsShadow.Create(AOwner: TPersistent);
begin
  inherited Create;
  FOwner := TsCustomStyle(AOwner);
  FColor := clBlack;
  FBlur := 8;
  FOffset := 8;
  FTransparency := 60;
end;

destructor TsShadow.Destroy;
begin
  if Assigned(FLabel) then begin
    FLabel.Free;
  end;
  inherited;
end;

procedure TsShadow.SetBlur(const Value: integer);
begin
  if FBlur <> Value then begin
    FBlur := Value;
    if Assigned(FLabel) then begin
      FLabel.Blur := Value;
      FOwner.AlignShadow;
    end;
  end;
end;

procedure TsShadow.SetColor(const Value: TColor);
begin
  if (FColor <> Value) then begin
    if Enabled and Assigned(FLabel) then begin
      FLabel.Color := Value;
    end;
    FColor := Value;
  end;
end;

procedure TsShadow.SetEnabled(const Value: boolean);
begin
  if GetStyleInfo(FOwner.FOwner.Parent) > 0 then begin // if Parent of Control - support sShadow
    if Value then begin
      if not Assigned(FLabel) and not DontUse and
         // If Parent - TsCustomPanel family, then dont show shadow?
         not ((FOwner.FOwner.Parent is TsCustomPanel) and
              not (FOwner.FOwner is TsCustomButton) and
              (TsCustomPanel(FOwner.FOwner.Parent).sStyle.Shadow.Enabled) and
              (TsCustomPanel(FOwner.FOwner.Parent).sStyle.Painting.Transparency > -1{!!!})) then begin

        FLabel := TsShadowing.Create(FOwner.FOwner);
        FLabel.Color := Color;
        FLabel.FBlur := Blur;
        FLabel.FTransparency := Transparency;
        FLabel.AutoSize := False;
        FLabel.Caption := ' ';
        FLabel.Name := FOwner.FOwner.Name + 'Shadow';
        FLabel.Parent := FOwner.FOwner.Parent;
        FLabel.Optimize := FOwner.Painting.Optimize;
        FEnabled := True;
        if FOwner.FOwner is TsRadioButton then FLabel.Shape := ssEllipse;
        FLabel.SendToBack;
        if FOwner.FOwner is TsCheckedControl then begin
          FOwner.FOwner.Invalidate;
        end
        else begin
          FOwner.AlignShadow;
        end;
      end;
    end
    else if Assigned(FLabel) and Enabled then begin
      FLabel.Free;
      FLabel := nil;
      FEnabled := False;
      if FOwner.FOwner is TsCheckedControl then FOwner.FOwner.Invalidate;
    end;
    FOwner.FOwner.Invalidate;
  end;
//  else ShowWarning('Shadow drawing possible on TsCustomPanel component family only.');
end;

procedure TsShadow.SetOffset(const Value: integer);
begin
  if (FOffset <> Value) then begin
    FOffset := Value;
    FOwner.AlignShadow;
  end;
end;

procedure TsShadow.SetTransparency(const Value: integer);
begin
  if FTransparency <> Value then begin
    if Value < 0 then FTransparency := 0
    else if Value > 100 then FTransparency := 100
    else FTransparency := Value;
    if Enabled and Assigned(FLabel) then begin
      FLabel.Transparency := FTransparency;
      FOwner.FOwner.Invalidate;
    end;
  end;
end;

{ TsData }

constructor TsData.Create(AOwner: TPersistent);
begin
  inherited Create;
  FOwner := TsStyle(AOwner);
  Allowing := TsAllowing.Create(AOwner);
end;

destructor TsData.Destroy;
begin
  Allowing.Free;
  inherited;
end;

procedure TsData.SetField(Value: TField);
begin
  if (FField <> Value) and Assigned(Value) then begin
    FField := Value;
    Allowing.FAutoInit := True;
    case FField.FieldKind of //   
      fkData: begin
//        Allowing.FInsert := True;
//        Allowing.FUpdate := True;
      end;
      fkCalculated, fkLookup: begin
        Allowing.Insert := False;
        Allowing.Update := False;
      end;
    end;
    Case Value.DataType of //   
      ftUnknown, ftString, ftBytes, ftVarBytes, ftBlob, ftMemo, ftGraphic, ftFmtMemo,
      ftParadoxOle, ftDBaseOle, ftTypedBinary, ftFixedChar, ftWideString,
      ftVariant: begin
        IsNumber := False;
{        if Between(FOwner.COC, COC_TsEdit, COC_TsEdit) and Assigned(Value) then begin
          TsEdit(FOwner).MaxLength := Field.Size;
        end else if Between(FOwner.COC, COC_TsComboEdit, COC_TsComboEdit) and Assigned(Value) then begin
          TsComboEdit(FOwner).MaxLength := Field.Size;
        end
        else
        if Between(FOwner.COC, COC_TsCustomComboBox, COC_TsBDEComboBox) and Assigned(Value) then begin
          TsCustomCombobox(FOwner).MaxLength := Field.Size;
        end;}
      end;
      ftSmallint, ftInteger, ftWord, ftBoolean, ftFloat, ftCurrency, ftBCD,
      ftAutoInc, ftLargeint: begin
        IsNumber := True;
      end;
      ftDate, ftTime, ftDateTime : begin
        Self.IsNumber := False;
      end
    end;
  end;
  inherited;
end;

{ TsPainting }

constructor TsPainting.Create(AOwner: TPersistent);
begin
  inherited Create;
  FColorBorderTop := clRed;
  FColorBorderBottom := clBlue;
  FColorActive := clWhite;
  FColorInActive := cl3DLight;
//  ParentColor := True;
  FOptimize := True;
  FOwner := TsCustomStyle(AOwner);
end;

procedure TsPainting.SetColors(Index: Integer; Value: TColor);
begin
  case Index of
    0: FColorBorderTop := Value;
    1: FColorBorderBottom := Value;
    2: FColorActive := Value;
    3: begin
      FColorInActive := Value;
      FParentColor := False;
    end;
  end;
  try
    if Assigned(FOwner) and Assigned(FOwner.FOwner) then FOwner.FOwner.Invalidate;
  except
  end;
end;

procedure TsPainting.SetOptimize(const Value: boolean);
begin
  FOptimize := Value;
  if Assigned(FOwner.Shadow.FLabel) then
    FOwner.Shadow.FLabel.Optimize := Value;
end;

procedure TsPainting.SetParentColor(Value: Boolean);
begin
  if Value <> FParentColor then begin
    FParentColor := Value;
    if FParentColor then begin
      if Assigned(FOwner.FOwner.Parent) and (FColorInActive <> TForm(FOwner.FOwner.Parent).Color) then begin
        FColorInActive := TForm(FOwner.FOwner.Parent).Color;
        FOwner.FOwner.invalidate;
      end;
    end;
  end;
end;

procedure TsPainting.SetTransparency(const Value: integer);
begin
  if FTransparency <> Value then begin
    FTransparency := Value;
    FOwner.FOwner.invalidate;
  end;
end;

{ TsStyle }

constructor TsStyle.Create(AOwner: TWinControl);
begin
  inherited Create(AOwner);
  Data := TsData.Create(Self);
end;

procedure TsStyle.WndProc(var Message: TMessage);
begin
  if (Message.Msg >= SM_OFFSET) and (Message.Msg <= SM_LAST) then begin
    if Message.Msg = SM_GETSTYLEINFO then begin
      TSMGetStyleInfo(Message).WParam := tos_SSTYLE;
      TSMGetStyleInfo(Message).LParam := Longint(Self);
    end
    else sStyleMessage(Message);
  end
  else inherited;
end;

procedure TsStyle.sStyleMessage(var Message: TMessage);
begin
  inherited;
end;

procedure TsStyleDB.WndProc(var Message: TMessage);
begin
  if (Message.Msg >= SM_OFFSET) and (Message.Msg <= SM_LAST) then begin
    if Message.Msg = SM_GETSTYLEINFO then begin
      TSMGetStyleInfo(Message).WParam := tos_SSTYLEDB;
      TSMGetStyleInfo(Message).LParam := Longint(Self);
    end else sStyleMessage(Message);
  end
  else inherited;
end;

procedure TsStyleDB.sStyleMessage(var Message: TMessage);
begin
  inherited;
end;

{ TsBackground }

constructor TsBgControl.Create(AOwner: TsStyleControl);
begin
  inherited Create(AOwner);
  FOwner := AOwner;
end;

procedure TsBgControl.BkGroundsUpdate;
begin
  inherited;
  if FOwner.Active then ChangeBkGround(FOwner.FForm, Self);
end;

{ TsBackground }

procedure TsBackground.BkGroundsUpdate;
begin
  if Between(FOwner.COC, COC_TsPanel, COC_TsCustomPanel) then begin
    FOwner.FOwner.Invalidate;
  end;
end;

constructor TsBackground.Create(AOwner: TsPaintStyle);
begin
//  inherited Create(AOwner);
  FOwner := AOwner;
  FImage := TPicture.Create;
  FGradient := TPassiveGradient.Create(Self);
  FMode := bgsGradient;
end;

{ TsPassiveBackground }

constructor TsPassiveBackground.Create(AOwner: TPersistent);
begin
  inherited Create;
  FOwner := TsPaintStyle(AOwner);
  FImage := TPicture.Create;
  FGradient := TPassiveGradient.Create(Self);
  FMode := bgsGradient;
end;

destructor TsPassiveBackground.Destroy;
begin
  FImage.Free;
  FGradient.Free;
  inherited;
end;

procedure TsPassiveBackground.BkGroundsUpdate;
begin
//  FOwner.FOwner.Invalidate;
end;

procedure TsPassiveBackground.SetImage(const Value: TPicture);
begin
  if FImage <> Value then begin
    FImage.Assign(Value);
    BkGroundsUpdate;
  end;
end;

procedure TsPassiveBackground.SetStyle(const Value: TBgStyle);
begin
  if FMode <> Value then begin
    FMode := Value;
    BkGroundsUpdate;
  end;
end;

{ TPassiveGradient }

constructor TPassiveGradient.Create(AOwner: TsPassiveBackground);
begin
  FOwner := TsPassiveBackground(AOwner);
  FLayout := gtTopBottom;
  FColor1 := $00D0CDB9;
  FColor2 := clSilver;
end;

procedure TPassiveGradient.SetColors(Index: Integer; Value: TColor);
begin
  case Index of
    0: FColor1 := Value;
    1: FColor2 := Value;
  end;
  FOwner.BkGroundsUpdate;
end;

procedure TPassiveGradient.SetLayout(const Value: TGradientTypes);
begin
  FLayout := Value;
  FOwner.BkGroundsUpdate;
end;

{ TsPassiveBGStyle }

constructor TsPassiveBGStyle.Create(AOwner: TWinControl);
begin
  inherited Create(AOwner);
  FBackground := TsBackground.Create(Self);
  sC := GetsStyleControl(sVclUtils.GetParentForm(AOwner), GroupIndex);
  if Assigned(sC) then begin
    FBackGround.Mode := sC.Background.Mode;
    case FBackGround.Mode of
      bgsGradient : begin
        FBackGround.Gradient.Color1 := sC.Background.Gradient.Color1;
        FBackGround.Gradient.Color2 := sC.Background.Gradient.Color2;
        FBackGround.Gradient.Layout := sC.Background.Gradient.Layout;
      end;
      bgsImage : begin
        FBackGround.Image.Assign(sC.Background.Image);
      end;
    end;
  end;
end;

procedure TsPaintStyle.PaintWindow(Canvas: TCanvas; Left, Top, Width, Height : integer; Text: string; Enabled: boolean; Flags: Longint);
begin
  Canvas.Brush.Color := GetActiveColor;
  Canvas.Brush.Style := bsSolid;
  Canvas.Rectangle(Left, Top, Left + Width, Top + Height);
  WriteText(Canvas, PChar(Text), Enabled, Left, Top, Width, Height, Flags); 
end;

procedure TsPaintStyle.WriteText(Canvas: TCanvas; Text: PChar; Enabled: boolean; Left, Top, Width, Height : integer; Flags: Longint);
var
  R, Rd: TRect;
  x : integer;
begin
  R := Rect(Left, Top, Left + Width, Top + Height);
  x := (Canvas.TextHeight('W') - Height) div 2;
  InflateRect(R, x, x);           

  Canvas.Brush.Style := bsClear;
  if Enabled then begin
    DrawText(Canvas.Handle, Text, -1, R, Flags);
  end
  else begin
    Rd := Rect(Left + 1, Top + 1, Left + Width + 1, Top + Height + 1);
    Canvas.Font.Color := clBtnHighlight;
    DrawText(Canvas.Handle, Text, -1, Rd, Flags);
    Canvas.Font.Color := clBtnShadow;
    DrawText(Canvas.Handle, Text, -1, R, Flags);
  end;
end;

end.
